#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iOS sysdiagnose logd files (logd.0.log) text parser plugin."""

import unittest

from plaso.parsers.text_plugins import ios_logd

from tests.parsers.text_plugins import test_lib


class IOSSysdiagnoseLogdUnitTest(test_lib.TextPluginTestCase):
  """Tests for the iOS sysdiagnose logd files (logd.0.log) text plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = ios_logd.IOSSysdiagnoseLogdTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['logd.0.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 76)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': 'libtrace_kic=1',
        'data_type': 'ios:sysdiagnose:logd:line',
        'logger': 'logd[29]',
        'written_time': '2021-08-11T05:50:23-07:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Plugin to parse the OLECF summary/document summary information items."""

from dfdatetime import filetime as dfdatetime_filetime

import pyolecf

from plaso.containers import events
from plaso.parsers import olecf
from plaso.parsers.olecf_plugins import interface


class OLECFDocumentSummaryInformationEventData(events.EventData):
  """OLECF document summary information event data.

  Attributes:
    application_version (str): application version.
    category (str): category of the document, such as memo or proposal.
    codepage (str): codepage of the document summary information.
    company (str): name of the company of the document.
    content_status (str): content status.
    content_type (str): content type.
    document_parts (list[str]): names of document parts.
    document_version (int): Version of the document.
    item_creation_time (dfdatetime.DateTimeValues): creation date and time of
        the item.
    item_modification_time (dfdatetime.DateTimeValues): modification date and
        time of the item.
    language (str): Language of the document.
    links_up_to_date (bool): True if the links are up to date.
    manager (str): name of the manager of the document.
    number_of_bytes (int): size of the document in bytes.
    number_of_characters_with_white_space (int): number of characters including
        spaces in the document.
    number_of_clips (int): number of multi-media clips in the document.
    number_of_hidden_slides (int): number of hidden slides in the document.
    number_of_lines (int): number of lines in the document.
    number_of_notes (int): number of notes in the document.
    number_of_paragraphs (int): number of paragraphs in the document.
    number_of_slides (int): number of slides in the document.
    presentation_format (str): target format for presentation, such as 35mm,
        printer or video.
    scale (bool): True if scaling of the thumbnail is desired or false if
        cropping is desired.
    shared_document (bool): True if the document is shared.
  """

  DATA_TYPE = 'olecf:document_summary_info'

  def __init__(self):
    """Initializes event data."""
    super(OLECFDocumentSummaryInformationEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.application_version = None
    self.category = None
    self.codepage = None
    self.company = None
    self.content_status = None
    self.content_type = None
    self.document_parts = None
    self.document_version = None
    self.item_creation_time = None
    self.item_modification_time = None
    self.language = None
    self.links_up_to_date = None
    self.manager = None
    self.number_of_bytes = None
    self.number_of_characters_with_white_space = None
    self.number_of_clips = None
    self.number_of_hidden_slides = None
    self.number_of_lines = None
    self.number_of_notes = None
    self.number_of_paragraphs = None
    self.number_of_slides = None
    self.presentation_format = None
    self.scale = None
    self.shared_document = None


class OLECFSummaryInformationEventData(events.EventData):
  """OLECF summary information event data.

  Attributes:
    application (str): name of application that created document.
    author (str): author of the document.
    codepage (str): codepage of the summary information.
    comments (str): comments.
    creation_time (dfdatetime.DateTimeValues): creation date and time of
        the document.
    edit_duration (int): total editing time.
    item_creation_time (dfdatetime.DateTimeValues): creation date and time of
        the item.
    item_modification_time (dfdatetime.DateTimeValues): modification date and
        time of the item.
    keywords (str): keywords.
    last_printed_time (dfdatetime.DateTimeValues): date and time the document
        was last printed.
    last_saved_by (str): name of user that last saved the document.
    last_save_time (dfdatetime.DateTimeValues): date and time the document was
        last saved.
    number_of_characters (int): number of characters without spaces in
        the document.
    number_of_pages (int): number of pages in the document.
    number_of_words (int): number of words in the document.
    revision_number (int): revision number.
    security_flags (int): security flags.
    subject (str): subject.
    template (str): name of the template used to created the document.
    title (str): title of the document.
  """

  DATA_TYPE = 'olecf:summary_info'

  def __init__(self):
    """Initializes event data."""
    super(OLECFSummaryInformationEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.application = None
    self.author = None
    self.codepage = None
    self.comments = None
    self.creation_time = None
    self.edit_duration = None
    self.item_creation_time = None
    self.item_modification_time = None
    self.keywords = None
    self.last_printed_time = None
    self.last_saved_by = None
    self.last_save_time = None
    self.number_of_characters = None
    self.number_of_pages = None
    self.number_of_words = None
    self.revision_number = None
    self.security_flags = None
    self.subject = None
    self.template = None
    self.title = None


class OLECFPropertySetStream(object):
  """OLECF property set stream.

  Attributes:
    date_time_properties (dict[str, dfdatetime.DateTimeValues]): date and time
        properties and values.
  """
  _CLASS_IDENTIFIER = None

  _INTEGER_TYPES = frozenset([
      pyolecf.value_types.INTEGER_16BIT_SIGNED,
      pyolecf.value_types.INTEGER_32BIT_SIGNED])

  _STRING_TYPES = frozenset([
      pyolecf.value_types.STRING_ASCII,
      pyolecf.value_types.STRING_UNICODE])

  _PROPERTY_NAMES = None
  _PROPERTY_VALUE_MAPPINGS = None

  def __init__(self, olecf_item):
    """Initialize an OLECF property set stream.

    Args:
      olecf_item (pyolecf.property_set_stream): OLECF item.
    """
    super(OLECFPropertySetStream, self).__init__()
    self._properties = {}
    self.date_time_properties = {}

    if olecf_item:
      self._ReadPropertySet(olecf_item.set)

  def _GetValueAsObject(self, property_value):
    """Retrieves the property value as a Python object.

    Args:
      property_value (pyolecf.property_value): OLECF property value.

    Returns:
      object: property value as a Python object.
    """
    if property_value.type == pyolecf.value_types.BOOLEAN:
      return property_value.data_as_boolean

    if property_value.type == pyolecf.value_types.FILETIME:
      filetime = property_value.data_as_integer
      if not filetime:
        return None

      return dfdatetime_filetime.Filetime(timestamp=filetime)

    if property_value.type in self._INTEGER_TYPES:
      return property_value.data_as_integer

    if property_value.type in self._STRING_TYPES:
      return property_value.data_as_string

    try:
      data = property_value.data
    except IOError:
      data = None

    return data

  def _ReadPropertySet(self, property_set):
    """Reads properties from a property set.

    Args:
      property_set (pyolecf.property_set): OLECF property set.
    """
    # Combine the values of multiple property sections
    # but do not override properties that are already set.
    for property_section in property_set.sections:
      if property_section.class_identifier != self._CLASS_IDENTIFIER:
        continue

      for property_value in property_section.properties:
        property_name = self._PROPERTY_NAMES.get(
            property_value.identifier, None)
        if not property_name:
          property_name = '0x{0:04}'.format(property_value.identifier)

        value = self._GetValueAsObject(property_value)
        if self._PROPERTY_VALUE_MAPPINGS:
          value_callback_name = self._PROPERTY_VALUE_MAPPINGS.get(
              property_name, None)
          if value_callback_name:
            value_callback_method = getattr(self, value_callback_name, None)
            if value_callback_method:
              value = value_callback_method(value)

        properties_dict = self._properties

        if property_name not in properties_dict:
          properties_dict[property_name] = value

  def SetEventData(self, event_data):
    """Sets the properties as event data.

    Args:
      event_data (EventData): event data.
    """
    for property_name, property_value in self._properties.items():
      if isinstance(property_value, bytes):
        property_value = repr(property_value)
      setattr(event_data, property_name, property_value)


class OLECFDocumentSummaryInformation(OLECFPropertySetStream):
  """OLECF Document Summary information property set."""

  _CLASS_IDENTIFIER = 'd5cdd502-2e9c-101b-9397-08002b2cf9ae'

  _PROPERTY_NAMES = {
      0x0001: 'codepage',  # PIDDSI_CODEPAGE
      0x0002: 'category',  # PIDDSI_CATEGORY
      0x0003: 'presentation_format',  # PIDDSI_PRESFORMAT
      0x0004: 'number_of_bytes',  # PIDDSI_BYTECOUNT
      0x0005: 'number_of_lines',  # PIDDSI_LINECOUNT
      0x0006: 'number_of_paragraphs',  # PIDDSI_PARCOUNT
      0x0007: 'number_of_slides',  # PIDDSI_SLIDECOUNT
      0x0008: 'number_of_notes',  # PIDDSI_NOTECOUNT
      0x0009: 'number_of_hidden_slides',  # PIDDSI_HIDDENCOUNT
      0x000a: 'number_of_clips',  # PIDDSI_MMCLIPCOUNT
      0x000b: 'scale',  # PIDDSI_SCALE
      # 0x000c: 'heading_pair',  # PIDDSI_HEADINGPAIR
      0x000d: 'document_parts',  # PIDDSI_DOCPARTS
      0x000e: 'manager',  # PIDDSI_MANAGER
      0x000f: 'company',  # PIDDSI_COMPANY
      0x0010: 'links_up_to_date',  # PIDDSI_LINKSDIRTY
      0x0011: 'number_of_characters_with_white_space',  # PIDDSI_CCHWITHSPACES
      0x0013: 'shared_document',  # PIDDSI_SHAREDDOC
      0x0017: 'application_version',  # PIDDSI_VERSION
      0x001a: 'content_type',  # PIDDSI_CONTENTTYPE
      0x001b: 'content_status',  # PIDDSI_CONTENTSTATUS
      0x001c: 'language',  # PIDDSI_LANGUAGE
      0x001d: 'document_version'}  # PIDDSI_DOCVERSION

  _PROPERTY_VALUE_MAPPINGS = {
      'application_version': '_FormatApplicationVersion'}

  def _FormatApplicationVersion(self, application_version):
    """Formats the application version.

    Args:
      application_version (int): application version.

    Returns:
      str: formatted application version.
    """
    # The application version consists of 2 16-bit values that make up
    # the version number. Where the upper 16-bit is the major number
    # and the lower 16-bit the minor number.
    return '{0:d}.{1:d}'.format(
        application_version >> 16, application_version & 0xffff)


class OLECFSummaryInformation(OLECFPropertySetStream):
  """OLECF Summary information property set."""

  _CLASS_IDENTIFIER = 'f29f85e0-4ff9-1068-ab91-08002b27b3d9'

  _PROPERTY_NAMES = {
      0x0001: 'codepage',  # PIDSI_CODEPAGE
      0x0002: 'title',  # PIDSI_TITLE
      0x0003: 'subject',  # PIDSI_SUBJECT
      0x0004: 'author',  # PIDSI_AUTHOR
      0x0005: 'keywords',  # PIDSI_KEYWORDS
      0x0006: 'comments',  # PIDSI_COMMENTS
      0x0007: 'template',  # PIDSI_TEMPLATE
      0x0008: 'last_saved_by',  # PIDSI_LASTAUTHOR
      0x0009: 'revision_number',  # PIDSI_REVNUMBER
      0x000a: 'edit_duration',  # PIDSI_EDITTIME
      0x000b: 'last_printed_time',  # PIDSI_LASTPRINTED
      0x000c: 'creation_time',  # PIDSI_CREATE_DTM
      0x000d: 'last_save_time',  # PIDSI_LASTSAVE_DTM
      0x000e: 'number_of_pages',  # PIDSI_PAGECOUNT
      0x000f: 'number_of_words',  # PIDSI_WORDCOUNT
      0x0010: 'number_of_characters',  # PIDSI_CHARCOUNT
      # 0x0011: 'thumbnail',  # PIDSI_THUMBNAIL
      0x0012: 'application',  # PIDSI_APPNAME
      0x0013: 'security_flags'}  # PIDSI_SECURITY


class DocumentSummaryInformationOLECFPlugin(interface.OLECFPlugin):
  """Plugin that parses DocumentSummaryInformation item from an OLECF file."""

  NAME = 'olecf_document_summary'
  DATA_FORMAT = (
      'Document summary information (\\0x05DocumentSummaryInformation)')

  # pylint: disable=anomalous-backslash-in-string
  REQUIRED_ITEMS = frozenset(['\005DocumentSummaryInformation'])

  def Process(self, parser_mediator, root_item=None, **kwargs):
    """Extracts events from a document summary information OLECF item.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      root_item (Optional[pyolecf.item]): root item of the OLECF file.

    Raises:
      ValueError: If the root item is not set.
    """
    # This will raise if unhandled keyword arguments are passed.
    super(DocumentSummaryInformationOLECFPlugin, self).Process(
        parser_mediator, **kwargs)

    if not root_item:
      raise ValueError('Root item not set.')

    for item_name in self.REQUIRED_ITEMS:
      item = root_item.get_sub_item_by_name(item_name)
      if item:
        event_data = OLECFDocumentSummaryInformationEventData()
        event_data.item_creation_time = self._GetCreationTime(root_item)
        event_data.item_modification_time = self._GetModificationTime(root_item)

        summary_information = OLECFDocumentSummaryInformation(item)
        summary_information.SetEventData(event_data)

        parser_mediator.ProduceEventData(event_data)


class SummaryInformationOLECFPlugin(interface.OLECFPlugin):
  """Plugin that parses the SummaryInformation item from an OLECF file."""

  NAME = 'olecf_summary'
  DATA_FORMAT = (
      'Summary information (\\0x05SummaryInformation) (top-level only)')

  # pylint: disable=anomalous-backslash-in-string
  REQUIRED_ITEMS = frozenset(['\005SummaryInformation'])

  def Process(self, parser_mediator, root_item=None, **kwargs):
    """Extracts events from a summary information OLECF item.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      root_item (Optional[pyolecf.item]): root item of the OLECF file.

    Raises:
      ValueError: If the root item is not set.
    """
    # This will raise if unhandled keyword arguments are passed.
    super(SummaryInformationOLECFPlugin, self).Process(
        parser_mediator, **kwargs)

    if not root_item:
      raise ValueError('Root item not set.')

    for item_name in self.REQUIRED_ITEMS:
      item = root_item.get_sub_item_by_name(item_name)
      if item:
        event_data = OLECFSummaryInformationEventData()
        event_data.item_creation_time = self._GetCreationTime(root_item)
        event_data.item_modification_time = self._GetModificationTime(root_item)

        summary_information = OLECFSummaryInformation(item)
        summary_information.SetEventData(event_data)

        parser_mediator.ProduceEventData(event_data)


olecf.OLECFParser.RegisterPlugins([
    DocumentSummaryInformationOLECFPlugin, SummaryInformationOLECFPlugin])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""The log2timeline command line tool."""

import logging
import multiprocessing
import os
import sys

from plaso import dependencies
from plaso.cli import log2timeline_tool
from plaso.lib import errors


def Main():
  """The main function."""
  tool = log2timeline_tool.Log2TimelineTool()

  if not tool.ParseArguments(sys.argv[1:]):
    return False

  if tool.show_troubleshooting:
    print('Using Python version {0!s}'.format(sys.version))
    print()
    print('Path: {0:s}'.format(os.path.abspath(__file__)))
    print()
    print(tool.GetVersionInformation())
    print()
    dependencies.CheckDependencies(verbose_output=True)

    print('Also see: https://plaso.readthedocs.io/en/latest/sources/user/'
          'Troubleshooting.html')
    return True

  try:
    tool.CheckOutDated()
  except KeyboardInterrupt:
    return False

  if tool.show_info:
    tool.ShowInfo()
    return True

  have_list_option = False
  if tool.list_archive_types:
    tool.ListArchiveTypes()
    have_list_option = True

  if tool.list_hashers:
    tool.ListHashers()
    have_list_option = True

  if tool.list_language_tags:
    tool.ListLanguageTags()
    have_list_option = True

  if tool.list_parsers_and_plugins:
    tool.ListParsersAndPlugins()
    have_list_option = True

  if tool.list_profilers:
    tool.ListProfilers()
    have_list_option = True

  if tool.list_time_zones:
    tool.ListTimeZones()
    have_list_option = True

  if have_list_option:
    return True

  if tool.dependencies_check and not dependencies.CheckDependencies(
      verbose_output=False):
    return False

  try:
    tool.ExtractEventsFromSources()

  # Writing to stdout and stderr will raise BrokenPipeError if it
  # receives a SIGPIPE.
  except BrokenPipeError:
    pass

  except (KeyboardInterrupt, errors.UserAbort):
    logging.warning('Aborted by user.')
    return False

  except (IOError, errors.BadConfigOption,
          errors.SourceScannerError) as exception:
    # Display message on stdout as well as the log file.
    print(exception)
    logging.error(exception)
    return False

  return True


if __name__ == '__main__':
  # For PyInstaller sake we need to define this directly after "__main__".
  # https://github.com/pyinstaller/pyinstaller/wiki/Recipe-Multiprocessing
  multiprocessing.freeze_support()

  if not Main():
    sys.exit(1)
  else:
    sys.exit(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Redis storage file reader."""

import unittest

try:
  # pylint: disable=ungrouped-imports
  import fakeredis
  import redis
  from plaso.storage.redis import reader
except ModuleNotFoundError:
  redis = None

from plaso.containers import sessions
from plaso.containers import tasks

from tests.storage import test_lib


@unittest.skipIf(redis is None, 'missing redis support')
class RedisStorageReaderTest(test_lib.StorageTestCase):
  """Tests for the Redis storage file reader."""

  _REDIS_URL = 'redis://127.0.0.1/0'

  def _CreateRedisClient(self):
    """Creates a Redis client for testing.

    This method will attempt to use a Redis server listening on localhost and
    fallback to a fake Redis client if no server is available or the connection
    timed out.

    Returns:
      Redis: a Redis client.
    """
    try:
      redis_client = redis.from_url(self._REDIS_URL, socket_timeout=60)
      redis_client.ping()
    except redis.exceptions.ConnectionError:
      redis_client = fakeredis.FakeStrictRedis()

    return redis_client

  def testInitialization(self):
    """Tests the __init__ function."""
    test_redis_client = self._CreateRedisClient()

    session = sessions.Session()
    task = tasks.Task(session_identifier=session.identifier)

    test_reader = reader.RedisStorageReader(
        session.identifier, task.identifier, redis_client=test_redis_client)
    self.assertIsNotNone(test_reader)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the output module field formatting helper."""

import platform
import unittest

from dfdatetime import posix_time as dfdatetime_posix_time
from dfdatetime import semantic_time as dfdatetime_semantic_time

from dfvfs.path import fake_path_spec

from plaso.containers import events
from plaso.lib import definitions
from plaso.output import formatting_helper

from tests.containers import test_lib as containers_test_lib
from tests.output import test_lib


class TestFieldFormattingHelper(formatting_helper.FieldFormattingHelper):
  """Field formatter helper for testing purposes."""

  _FIELD_FORMAT_CALLBACKS = {'zone': '_FormatTimeZone'}


class FieldFormattingHelperTest(test_lib.OutputModuleTestCase):
  """Test the output module field formatting helper."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'filename': 'log/syslog.1',
       'hostname': 'ubuntu',
       'path_spec': fake_path_spec.FakePathSpec(
           location='log/syslog.1'),
       'text': (
           'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session\n '
           'closed for user root)'),
       'timestamp': '2012-06-27 18:17:01',
       'timestamp_desc': definitions.TIME_DESCRIPTION_METADATA_MODIFICATION},
      {'data_type': 'test:event',
       'filename': 'log/syslog.1',
       'hostname': 'ubuntu',
       'path_spec': fake_path_spec.FakePathSpec(
           location='log/syslog.1'),
       'text': (
           'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session\n '
           'closed for user root)'),
       'timestamp': '2012-06-27 20:17:01',
       'timestamp_desc': definitions.TIME_DESCRIPTION_METADATA_MODIFICATION}]

  def testFormatDateTime(self):
    """Tests the _FormatDateTime function with dynamic time."""
    output_mediator = self._CreateOutputMediator(dynamic_time=True)
    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T18:17:01.000000+00:00')

    output_mediator.SetTimeZone('Europe/Amsterdam')

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T20:17:01.000000+02:00')

    output_mediator.SetTimeZone('UTC')

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[1]))
    event.date_time._time_zone_offset = 120

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T18:17:01.000000+00:00')

    event.date_time = dfdatetime_semantic_time.InvalidTime()

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, 'Invalid')

    # Test with event.is_local_time
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    event.timestamp -= 120 * 60 * 1000000
    event.date_time.is_local_time = True

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T16:17:01.000000+00:00')

  def testFormatDateTimeWithoutDynamicTime(self):
    """Tests the _FormatDateTime function without dynamic time."""
    output_mediator = self._CreateOutputMediator(dynamic_time=False)
    test_helper = formatting_helper.FieldFormattingHelper()

    # Test with event.date_time
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T18:17:01.000000+00:00')

    output_mediator.SetTimeZone('Europe/Amsterdam')

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T20:17:01.000000+02:00')

    output_mediator.SetTimeZone('UTC')

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[1]))
    event.date_time._time_zone_offset = 120

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T18:17:01.000000+00:00')

    event.date_time = dfdatetime_posix_time.PosixTimeInMilliseconds(
        timestamp=-1567517139327)

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    if platform.system() == 'Windows':
      expected_date_time_string = '0000-00-00T00:00:00.000000+00:00'
    else:
      expected_date_time_string = '1920-04-30T10:34:20.673000+00:00'
    self.assertEqual(date_time_string, expected_date_time_string)

    event.date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
        timestamp=-1567517139327447)

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    if platform.system() == 'Windows':
      expected_date_time_string = '0000-00-00T00:00:00.000000+00:00'
    else:
      expected_date_time_string = '1920-04-30T10:34:20.672553+00:00'
    self.assertEqual(date_time_string, expected_date_time_string)

    event.date_time = dfdatetime_posix_time.PosixTimeInNanoseconds(
        timestamp=-1567517139327447871)

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    if platform.system() == 'Windows':
      expected_date_time_string = '0000-00-00T00:00:00.000000+00:00'
    else:
      expected_date_time_string = '1920-04-30T10:34:20.672552+00:00'
    self.assertEqual(date_time_string, expected_date_time_string)

    event.date_time = dfdatetime_semantic_time.InvalidTime()

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '0000-00-00T00:00:00.000000+00:00')

    # Test with event.is_local_time
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    event.timestamp -= 120 * 60 * 1000000
    event.date_time.is_local_time = True

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T16:17:01.000000+00:00')

    # Test with event.timestamp
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    event.date_time = None

    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '2012-06-27T18:17:01.000000+00:00')

    event.timestamp = 0
    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '0000-00-00T00:00:00.000000+00:00')

    event.timestamp = -1567517139327447
    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    if platform.system() == 'Windows':
      expected_date_time_string = '0000-00-00T00:00:00.000000+00:00'
    else:
      expected_date_time_string = '1920-04-30T10:34:20.672553+00:00'
    self.assertEqual(date_time_string, expected_date_time_string)

    event.timestamp = -9223372036854775808
    date_time_string = test_helper._FormatDateTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(date_time_string, '0000-00-00T00:00:00.000000+00:00')

  def testFormatDisplayName(self):
    """Tests the _FormatDisplayName function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    display_name_string = test_helper._FormatDisplayName(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(display_name_string, 'FAKE:log/syslog.1')

  def testFormatFilename(self):
    """Tests the _FormatFilename function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    filename_string = test_helper._FormatFilename(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(filename_string, 'log/syslog.1')

  def testFormatHostname(self):
    """Tests the _FormatHostname function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    hostname_string = test_helper._FormatHostname(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(hostname_string, 'ubuntu')

  def testFormatInode(self):
    """Tests the _FormatInode function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    inode_string = test_helper._FormatInode(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(inode_string, '-')

  def testFormatMACB(self):
    """Tests the _FormatMACB function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    macb_string = test_helper._FormatMACB(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(macb_string, '..C.')

  def testFormatMessage(self):
    """Tests the _FormatMessage function."""
    output_mediator = self._CreateOutputMediator()

    formatters_directory_path = self._GetTestFilePath(['formatters'])
    output_mediator.ReadMessageFormattersFromDirectory(
        formatters_directory_path)

    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    message_string = test_helper._FormatMessage(
        output_mediator, event, event_data, event_data_stream)

    expected_message_string = (
        'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session closed '
        'for user root)')
    self.assertEqual(message_string, expected_message_string)

  def testFormatMessageShort(self):
    """Tests the _FormatMessageShort function."""
    output_mediator = self._CreateOutputMediator()

    formatters_directory_path = self._GetTestFilePath(['formatters'])
    output_mediator.ReadMessageFormattersFromDirectory(
        formatters_directory_path)

    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    message_short_string = test_helper._FormatMessageShort(
        output_mediator, event, event_data, event_data_stream)

    expected_message_short_string = (
        'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session closed '
        'for user root)')
    self.assertEqual(message_short_string, expected_message_short_string)

  def testFormatSource(self):
    """Tests the _FormatSource function."""
    output_mediator = self._CreateOutputMediator()

    formatters_directory_path = self._GetTestFilePath(['formatters'])
    output_mediator.ReadMessageFormattersFromDirectory(
        formatters_directory_path)

    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    source_string = test_helper._FormatSource(
        output_mediator, event, event_data, event_data_stream)

    self.assertEqual(source_string, 'Test log file')

  def testFormatSourceShort(self):
    """Tests the _FormatSourceShort function."""
    output_mediator = self._CreateOutputMediator()

    formatters_directory_path = self._GetTestFilePath(['formatters'])
    output_mediator.ReadMessageFormattersFromDirectory(
        formatters_directory_path)

    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    source_short_string = test_helper._FormatSourceShort(
        output_mediator, event, event_data, event_data_stream)

    self.assertEqual(source_short_string, 'FILE')

  def testFormatTag(self):
    """Tests the _FormatTag function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    tag_string = test_helper._FormatTag(output_mediator, None)
    self.assertEqual(tag_string, '-')

    event_tag = events.EventTag()
    event_tag.AddLabel('one')
    event_tag.AddLabel('two')

    tag_string = test_helper._FormatTag(output_mediator, event_tag)
    self.assertEqual(tag_string, 'one two')

  def testFormatTime(self):
    """Tests the _FormatTime function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    # Test with event.date_time
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    time_string = test_helper._FormatTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(time_string, '18:17:01')

    output_mediator.SetTimeZone('Europe/Amsterdam')

    time_string = test_helper._FormatTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(time_string, '20:17:01')

    output_mediator.SetTimeZone('UTC')

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[1]))
    event.date_time._time_zone_offset = 120

    time_string = test_helper._FormatTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(time_string, '18:17:01')

    # Test with event.is_local_time
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    event.timestamp -= 120 * 60 * 1000000
    event.date_time.is_local_time = True

    time_string = test_helper._FormatTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(time_string, '16:17:01')

    # Test with event.timestamp
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    event.date_time = None

    time_string = test_helper._FormatTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(time_string, '18:17:01')

    event.timestamp = 0
    time_string = test_helper._FormatTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(time_string, '--:--:--')

    event.timestamp = -9223372036854775808
    time_string = test_helper._FormatTime(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(time_string, '--:--:--')

  def testFormatTimeZone(self):
    """Tests the _FormatTimeZone function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    # Test with event.date_time
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))

    zone_string = test_helper._FormatTimeZone(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(zone_string, 'UTC')

    output_mediator.SetTimeZone('Europe/Amsterdam')

    zone_string = test_helper._FormatTimeZone(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(zone_string, 'CEST')

    output_mediator.SetTimeZone('UTC')

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[1]))
    event.date_time._time_zone_offset = 120

    zone_string = test_helper._FormatTimeZone(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(zone_string, 'UTC')

    # Test with event.is_local_time
    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    event.timestamp -= 120 * 60 * 1000000
    event.date_time.is_local_time = True

    zone_string = test_helper._FormatTimeZone(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(zone_string, 'UTC')

  def testFormatUsername(self):
    """Tests the _FormatUsername function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = formatting_helper.FieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    username_string = test_helper._FormatUsername(
        output_mediator, event, event_data, event_data_stream)
    self.assertEqual(username_string, '-')

  # TODO: add coverage for _ReportEventError

  def testGetFormattedField(self):
    """Tests the GetFormattedField function."""
    output_mediator = self._CreateOutputMediator()
    test_helper = TestFieldFormattingHelper()

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    zone_string = test_helper.GetFormattedField(
        output_mediator, 'zone', event, event_data, event_data_stream, None)
    self.assertEqual(zone_string, 'UTC')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Systemd Journal parser."""

import unittest

from plaso.containers import warnings
from plaso.parsers import systemd_journal

from tests.parsers import test_lib


class SystemdJournalParserTest(test_lib.ParserTestCase):
  """Tests for the Systemd Journal parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = systemd_journal.SystemdJournalParser()
    storage_writer = self._ParseFile([
        'systemd', 'journal', 'system.journal'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2101)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': 'Started User Manager for UID 1000.',
        'data_type': 'systemd:journal',
        'hostname': 'test-VirtualBox',
        'pid': '1',
        'reporter': 'systemd',
        'written_time': '2017-01-27T09:40:55.913258+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Test a XZ compressed data log entry.
    expected_event_values = {
        'body': 'a' * 692,
        'data_type': 'systemd:journal',
        'hostname': 'test-VirtualBox',
        'pid': '22921',
        'reporter': 'root',
        'written_time': '2017-02-06T16:24:32.564585+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2098)
    self.CheckEventData(event_data, expected_event_values)

  def testParseLZ4(self):
    """Tests the Parse function on a journal with LZ4 compressed events."""
    parser = systemd_journal.SystemdJournalParser()
    storage_writer = self._ParseFile([
        'systemd', 'journal', 'system.journal.lz4'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 85)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': 'Reached target Paths.',
        'data_type': 'systemd:journal',
        'hostname': 'testlol',
        'pid': '822',
        'reporter': 'systemd',
        'written_time': '2018-07-03T15:00:16.682340+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Test a LZ4 compressed data log entry.
    # The text used in the test message was triplicated to make it long enough
    # to trigger the LZ4 compression.
    # Source: https://github.com/systemd/systemd/issues/6237
    expected_body_parts = [' textual user names.']
    expected_body_parts.extend(
        ('  Yes, as you found out 0day is not a valid username. I wonder which '
         'tool permitted you to create it in the first place. Note that not '
         'permitting numeric first characters is done on purpose: to avoid '
         'ambiguities between numeric UID and textual user names.') * 3)
    expected_body = ''.join(expected_body_parts)

    expected_event_values = {
        'body': expected_body,
        'data_type': 'systemd:journal',
        'hostname': 'testlol',
        'pid': '34757',
        'reporter': 'test',
        'written_time': '2018-07-03T15:19:04.667807+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 84)
    self.CheckEventData(event_data, expected_event_values)

  def testParseDirty(self):
    """Tests the Parse function on a 'dirty' journal file."""
    parser = systemd_journal.SystemdJournalParser()

    storage_writer = self._ParseFile([
        'systemd', 'journal',
        'system@00053f9c9a4c1e0e-2e18a70e8b327fed.journalTILDE'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2211)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': ('Runtime journal (/run/log/journal/) is 1.2M, max 9.9M, 8.6M '
                 'free.'),
        'data_type': 'systemd:journal',
        'hostname': 'test-VirtualBox',
        'pid': '569',
        'reporter': 'systemd-journald',
        'written_time': '2016-10-24T13:20:01.063423+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    generator = storage_writer.GetAttributeContainers(
        warnings.ExtractionWarning.CONTAINER_TYPE)

    test_warnings = list(generator)
    test_warning = test_warnings[0]
    self.assertIsNotNone(test_warning)

    expected_message = (
        'Unable to parse journal entry at offset: 0x0041bfb0 with error: '
        'object offset should be after hash tables (0 < 2527472)')
    self.assertEqual(test_warning.message, expected_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*
"""Tests for the Azure Application Gateway Access log files parser."""

import unittest

from plaso.parsers.jsonl_plugins import azure_application_gateway_log

from tests.parsers.jsonl_plugins import test_lib


class AzureApplicationGatewayAccessLogJSONLPluginTest(
    test_lib.JSONLPluginTestCase):
  """Tests for the Azure Application Gateway Access log files parser."""

  def testProcess(self):
    """Tests the Process function."""
    plugin_module = azure_application_gateway_log
    plugin = plugin_module.AzureApplicationGatewayAccessLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(
        ['azure_application_gateway_access.json'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'client_ip': '185.42.129.24',
        'client_port': 45057,
        'client_response_time': 0,
        'data_type': 'azure:application_gateway_access:entry',
        'host': '20.110.30.194',
        'http_method': 'GET',
        'http_status': 200,
        'http_version': 'HTTP/1.1',
        'instance_identifier': 'appgw_2',
        'original_host': '20.110.30.194',
        'original_request_uri': '/',
        'received_bytes': 184,
        'recorded_time': '2021-10-14T22:17:11.000000+00:00',
        'request_uri': '/',
        'sent_bytes': 466,
        'server_response_latency': '0.028',
        'server_routed': '52.239.221.65:443',
        'server_status': '200',
        'ssl_cipher': 'ECDHE-RSA-AES256-GCM-SHA384',
        'ssl_client_verify': 'NONE',
        'ssl_enabled': 'on',
        'ssl_protocol': 'TLSv1.2',
        'time_taken': 0.034,
        'transaction_identifier': '592d1649f75a8d480a3c4dc6a975309d',
        'user_agent': (
            'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 '
            '(KHTML, like Gecko) Chrome/52.0.2743.116 Safari/537.36'),
        'waf_evaluation_time': '0.000',
        'waf_mode': 'Detection'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Kodi videos plugin."""

import unittest

from plaso.parsers.sqlite_plugins import kodi

from tests.parsers.sqlite_plugins import test_lib


class KodiVideosTest(test_lib.SQLitePluginTestCase):
  """Tests for the Kodi videos database plugin."""

  def testProcess(self):
    """Test the Process function on a Kodi Videos database."""
    plugin = kodi.KodiMyVideosPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['MyVideos107.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'kodi:videos:viewing',
        'filename': 'plugin://plugin.video.youtube/play/?video_id=7WX0-O_ENlk',
        'last_played_time': '2017-07-16T04:54:54+00:00',
        'play_count': 1}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Imports for the SQLite database parser plugins."""

from plaso.parsers.sqlite_plugins import android_calls
from plaso.parsers.sqlite_plugins import android_hangouts
from plaso.parsers.sqlite_plugins import android_sms
from plaso.parsers.sqlite_plugins import android_tango
from plaso.parsers.sqlite_plugins import android_twitter
from plaso.parsers.sqlite_plugins import android_webview
from plaso.parsers.sqlite_plugins import android_webviewcache
from plaso.parsers.sqlite_plugins import chrome_autofill
from plaso.parsers.sqlite_plugins import chrome_cookies
from plaso.parsers.sqlite_plugins import chrome_extension_activity
from plaso.parsers.sqlite_plugins import chrome_history
from plaso.parsers.sqlite_plugins import dropbox
from plaso.parsers.sqlite_plugins import firefox_cookies
from plaso.parsers.sqlite_plugins import firefox_downloads
from plaso.parsers.sqlite_plugins import firefox_history
from plaso.parsers.sqlite_plugins import gdrive
from plaso.parsers.sqlite_plugins import imessage
from plaso.parsers.sqlite_plugins import ios_datausage
from plaso.parsers.sqlite_plugins import ios_kik
from plaso.parsers.sqlite_plugins import ios_netusage
from plaso.parsers.sqlite_plugins import ios_powerlog
from plaso.parsers.sqlite_plugins import ios_screentime
from plaso.parsers.sqlite_plugins import ios_twitter
from plaso.parsers.sqlite_plugins import kodi
from plaso.parsers.sqlite_plugins import ls_quarantine
from plaso.parsers.sqlite_plugins import mackeeper_cache
from plaso.parsers.sqlite_plugins import macos_appusage
from plaso.parsers.sqlite_plugins import macos_document_versions
from plaso.parsers.sqlite_plugins import macos_knowledgec
from plaso.parsers.sqlite_plugins import macos_notes
from plaso.parsers.sqlite_plugins import macos_notification_center
from plaso.parsers.sqlite_plugins import macos_tcc
from plaso.parsers.sqlite_plugins import safari
from plaso.parsers.sqlite_plugins import skype
from plaso.parsers.sqlite_plugins import windows_eventtranscript
from plaso.parsers.sqlite_plugins import windows_timeline
from plaso.parsers.sqlite_plugins import zeitgeist



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Hangouts messages plugin."""

import unittest

from plaso.parsers.sqlite_plugins import android_hangouts

from tests.parsers.sqlite_plugins import test_lib


class AndroidHangoutsMessagesTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Hangouts message database plugin."""

  def testProcess(self):
    """Test the Process function on a Google hangouts file."""
    plugin = android_hangouts.AndroidHangoutsMessagePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['googlehangouts.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 14)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': 'How are you?',
        'creation_time': '2017-07-17T04:41:54.326967+00:00',
        'data_type': 'android:messaging:hangouts',
        'message_status': 4,
        'message_type': 2,
        'sender': 'John Macron'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains a WinRAR history Windows Registry plugin."""

import re

from plaso.containers import events
from plaso.parsers import winreg_parser
from plaso.parsers.winreg_plugins import interface


class WinRARHistoryEventData(events.EventData):
  """WinRAR history event data attribute container.

  Attributes:
    entries (str): archive history entries.
    key_path (str): Windows Registry key path.
    last_written_time (dfdatetime.DateTimeValues): entry last written date and
        time.
  """

  DATA_TYPE = 'winrar:history'

  def __init__(self):
    """Initializes event data."""
    super(WinRARHistoryEventData, self).__init__(data_type=self.DATA_TYPE)
    self.entries = None
    self.key_path = None
    self.last_written_time = None


class WinRARHistoryPlugin(interface.WindowsRegistryPlugin):
  """Windows Registry plugin for parsing WinRAR History keys."""

  NAME = 'winrar_mru'
  DATA_FORMAT = 'WinRAR History Registry data'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_CURRENT_USER\\Software\\WinRAR\\ArcHistory'),
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_CURRENT_USER\\Software\\WinRAR\\DialogEditHistory\\ArcName'),
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_CURRENT_USER\\Software\\WinRAR\\DialogEditHistory\\ExtrPath')])

  _RE_VALUE_NAME = re.compile(r'^[0-9]+$', re.I)

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    entries = []
    for registry_value in registry_key.GetValues():
      # Ignore any value not in the form: '[0-9]+'.
      if (not registry_value.name or
          not self._RE_VALUE_NAME.search(registry_value.name)):
        continue

      # Ignore any value that is empty or that does not contain a string.
      if not registry_value.data or not registry_value.DataIsString():
        continue

      value_string = registry_value.GetDataAsObject()
      entries.append('{0:s}: {1:s}'.format(registry_value.name, value_string))

    event_data = WinRARHistoryEventData()
    event_data.entries = ' '.join(entries) or None
    event_data.key_path = registry_key.path
    event_data.last_written_time = registry_key.last_written_time

    parser_mediator.ProduceEventData(event_data)


winreg_parser.WinRegistryParser.RegisterPlugin(WinRARHistoryPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows EventLog (EVT) parser."""

import unittest

from plaso.parsers import winevt

from tests.parsers import test_lib


class WinEvtParserTest(test_lib.ParserTestCase):
  """Tests for the Windows EventLog (EVT) parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = winevt.WinEvtParser()
    storage_writer = self._ParseFile(['SysEvent.Evt'], parser)

    # Windows Event Log (EVT) information:
    #	Version                     : 1.1
    #	Number of records           : 6063
    #	Number of recovered records : 438
    #	Log type                    : System

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6501)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Event number      : 1392
    # Creation time     : Jul 27, 2011 06:41:47 UTC
    # Written time      : Jul 27, 2011 06:41:47 UTC
    # Event type        : Warning event (2)
    # Computer name     : WKS-WINXP32BIT
    # Source name       : LSASRV
    # Event category    : 3
    # Event identifier  : 0x8000a001 (2147524609)
    # Number of strings : 2
    # String: 1         : cifs/CONTROLLER
    # String: 2         : "The system detected a possible attempt to compromise
    #                     security. Please ensure that you can contact the
    #                     server that authenticated you.\r\n (0xc0000388)"

    expected_event_values = {
        'creation_time': '2011-07-27T06:41:47+00:00',
        'computer_name': 'WKS-WINXP32BIT',
        'data_type': 'windows:evt:record',
        'event_category': 3,
        'event_identifier': 40961,
        'event_type': 2,
        'record_number': 1392,
        'severity': 2,
        'source_name': 'LSASRV',
        'strings': ['cifs/CONTROLLER', (
            '"The system detected a possible attempt to compromise security. '
            'Please ensure that you can contact the server that authenticated '
            'you.\r\n (0xc0000388)"')],
        'written_time': '2011-07-27T06:41:47+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The task-based multi-process processing engine."""

import os
import shutil
import tempfile

from plaso.lib import definitions
from plaso.multi_process import engine
from plaso.storage import factory as storage_factory

try:
  # pylint: disable=ungrouped-imports
  import redis
  from plaso.storage.redis import redis_store
except ModuleNotFoundError:
  redis = None
  redis_store = None


class TaskMultiProcessEngine(engine.MultiProcessEngine):
  """Task-based multi-process engine base.

  This class contains functionality to:
  * manage task storage used to store task results.
  """

  # pylint: disable=abstract-method

  def __init__(self):
    """Initializes a task-based multi-process engine."""
    super(TaskMultiProcessEngine, self).__init__()
    self._merge_task_storage_path = None
    self._processing_configuration = None
    self._processed_task_storage_path = None
    self._redis_client = None
    self._storage_file_path = None
    self._task_storage_path = None

  # TODO: remove, currently only used by psort.
  def _CheckTaskReadyForMerge(self, task_storage_format, task):
    """Checks if a task is ready for merging with this session storage.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Returns:
      bool: True if the task is ready to be merged.

    Raises:
      IOError: if the size of the SQLite task storage file cannot be determined.
      OSError: if the size of the SQLite task storage file cannot be determined.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      processed_storage_file_path = self._GetProcessedStorageFilePath(task)

      try:
        stat_info = os.stat(processed_storage_file_path)
      except (IOError, OSError):
        return False

      task.storage_file_size = stat_info.st_size
      return True

    return False

  def _GetMergeTaskStorage(self, task_storage_format, task):
    """Retrieves a task store ready to be merged with the session store.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Returns:
      StorageReader: storage reader of the task storage.

    Raises:
      IOError: if the temporary path for the task storage does not exist or
          if the temporary path for the task storage doe not refers to a file.
      OSError: if the temporary path for the task storage does not exist or
          if the temporary path for the task storage doe not refers to a file.
    """
    merge_storage_file_path = self._GetMergeTaskStorageFilePath(
        task_storage_format, task)

    if task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      if not self._merge_task_storage_path:
        raise IOError('Missing merge task storage path.')

      if not os.path.isfile(merge_storage_file_path):
        raise IOError('Merge task storage path is not a file.')

    task_storage_reader = (
        storage_factory.StorageFactory.CreateTaskStorageReader(
            task_storage_format, task, merge_storage_file_path))
    task_storage_reader.SetStorageProfiler(self._storage_profiler)

    return task_storage_reader

  def _GetMergeTaskStorageRedisHashName(self, task):
    """Retrieves the Redis hash name of a task store that should be merged.

    Args:
      task (Task): task the storage changes are part of.

    Returns:
      str: Redis hash name of a task store.
    """
    return '{0:s}-merge'.format(task.session_identifier)

  def _GetMergeTaskStorageFilePath(self, task_storage_format, task):
    """Retrieves the path of a task storage file in the merge directory.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Returns:
      str: path of a task storage file file in the merge directory or None if
          not set.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      filename = '{0:s}.plaso'.format(task.identifier)
      return os.path.join(self._merge_task_storage_path, filename)

    return None

  def _GetProcessedRedisHashName(self, session_identifier):
    """Retrieves the Redis hash name of a processed task store.

    Args:
      session_identifier (str): the identifier of the session the tasks are
          part of.

    Returns:
      str: Redis hash name of a task store.
    """
    return '{0:s}-processed'.format(session_identifier)

  def _GetProcessedStorageFilePath(self, task):
    """Retrieves the path of a task storage file in the processed directory.

    Args:
      task (Task): task the storage changes are part of.

    Returns:
      str: path of a task storage file in the processed directory.
    """
    filename = '{0:s}.plaso'.format(task.identifier)
    return os.path.join(self._processed_task_storage_path, filename)

  def _GetProcessedTaskIdentifiers(
      self, task_storage_format, session_identifier):
    """Identifiers for tasks which have been processed.

    Args:
      task_storage_format (str): storage format used to store task results.
      session_identifier (str): the identifier of the session the tasks are
          part of.

    Returns:
      list[str]: task identifiers that are processed.

    Raises:
      IOError: if the temporary path for the task storage does not exist.
      OSError: if the temporary path for the task storage does not exist.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_REDIS:
      redis_hash_name = self._GetProcessedRedisHashName(session_identifier)

      try:
        task_identifiers = self._redis_client.hkeys(redis_hash_name)
        task_identifiers = [
            identifier.decode('utf-8') for identifier in task_identifiers]
      except redis.exceptions.TimeoutError:
        # If there is a timeout fetching identifiers, we assume that there are
        # no processed tasks.
        task_identifiers = []

    elif task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      if not self._processed_task_storage_path:
        raise IOError('Missing processed task storage path.')

      task_identifiers = [
          path.replace('.plaso', '')
          for path in os.listdir(self._processed_task_storage_path)]

    return task_identifiers

  def _PrepareMergeTaskStorage(self, task_storage_format, task):
    """Prepares a task storage for merging.

    Moves the task storage file from the processed directory to the merge
    directory.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Raises:
      IOError: if the SQLite task storage file cannot be renamed.
      OSError: if the SQLite task storage file cannot be renamed.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_REDIS:
      # TODO: use number of attribute containers instead of file size?
      task.storage_file_size = 1000

      redis_hash_name = self._GetProcessedRedisHashName(task.session_identifier)
      number_of_results = self._redis_client.hdel(
          redis_hash_name, task.identifier)
      if number_of_results == 0:
        raise IOError('Task identifier {0:s} was not processed'.format(
            task.identifier))

      redis_hash_name = self._GetMergeTaskStorageRedisHashName(task)
      # TODO: set timestamp as value.
      self._redis_client.hset(
          redis_hash_name, key=task.identifier, value=b'true')

    elif task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      merge_storage_file_path = self._GetMergeTaskStorageFilePath(
          task_storage_format, task)
      processed_storage_file_path = self._GetProcessedStorageFilePath(task)

      task.storage_file_size = os.path.getsize(processed_storage_file_path)

      try:
        os.rename(processed_storage_file_path, merge_storage_file_path)
      except OSError as exception:
        raise IOError((
            'Unable to rename task storage file: {0:s} with error: '
            '{1!s}').format(processed_storage_file_path, exception))

  def _RemoveMergeTaskStorage(self, task_storage_format, task):
    """Removes a merge task storage.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Raises:
      IOError: if a SQLite task storage file cannot be removed.
      OSError: if a SQLite task storage file cannot be removed.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_REDIS:
      redis_hash_pattern = '{0:s}-{1:s}-*'.format(
          task.session_identifier, task.identifier)

      for redis_hash_name in self._redis_client.keys(redis_hash_pattern):
        self._redis_client.delete(redis_hash_name)

    elif task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      merge_storage_file_path = self._GetMergeTaskStorageFilePath(
          task_storage_format, task)

      try:
        os.remove(merge_storage_file_path)
      except OSError as exception:
        raise IOError((
            'Unable to remove merge task storage file: {0:s} with error: '
            '{1!s}').format(merge_storage_file_path, exception))

  def _RemoveProcessedTaskStorage(self, task_storage_format, task):
    """Removes a processed task storage.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Raises:
      IOError: if a SQLite task storage file cannot be removed.
      OSError: if a SQLite task storage file cannot be removed.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_REDIS:
      redis_hash_pattern = '{0:s}-{1:s}-*'.format(
          task.session_identifier, task.identifier)

      for redis_hash_name in self._redis_client.keys(redis_hash_pattern):
        self._redis_client.delete(redis_hash_name)

    elif task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      processed_storage_file_path = self._GetProcessedStorageFilePath(task)

      try:
        os.remove(processed_storage_file_path)
      except OSError as exception:
        raise IOError((
            'Unable to remove processed task storage file: {0:s} with error: '
            '{1!s}').format(processed_storage_file_path, exception))

  def _StartTaskStorage(self, task_storage_format):
    """Starts the task storage.

    Args:
      task_storage_format (str): storage format used to store task results.

    Raises:
      IOError: if the temporary path for the SQLite task storage already exists.
      OSError: if the temporary path for the SQLite task storage already exists.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_REDIS and redis_store:
      url = redis_store.RedisStore.DEFAULT_REDIS_URL
      self._redis_client = redis.from_url(url=url, socket_timeout=60)
      self._redis_client.client_setname('task_engine')

    elif task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      if self._task_storage_path:
        raise IOError('SQLite task storage path already exists.')

      output_directory = os.path.dirname(self._storage_file_path)
      self._task_storage_path = tempfile.mkdtemp(dir=output_directory)

      self._merge_task_storage_path = os.path.join(
          self._task_storage_path, 'merge')
      os.mkdir(self._merge_task_storage_path)

      self._processed_task_storage_path = os.path.join(
          self._task_storage_path, 'processed')
      os.mkdir(self._processed_task_storage_path)

      self._processing_configuration.task_storage_path = self._task_storage_path

  def _StopTaskStorage(
      self, task_storage_format, session_identifier, abort=False):
    """Stops the task storage.

    The results of tasks will be lost on abort.

    Args:
      task_storage_format (str): storage format used to store task results.
      session_identifier (str): the identifier of the session the tasks are
          part of.
      abort (Optional[bool]): True to indicate the stop is issued on abort.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_REDIS:
      redis_hash_pattern = '{0:s}-*'.format(session_identifier)

      for redis_hash_name in self._redis_client.keys(redis_hash_pattern):
        self._redis_client.delete(redis_hash_name)

      self._redis_client = None

    elif task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      if os.path.isdir(self._merge_task_storage_path):
        if abort:
          shutil.rmtree(self._merge_task_storage_path)
        else:
          os.rmdir(self._merge_task_storage_path)

      if os.path.isdir(self._processed_task_storage_path):
        if abort:
          shutil.rmtree(self._processed_task_storage_path)
        else:
          os.rmdir(self._processed_task_storage_path)

      if os.path.isdir(self._task_storage_path):
        if abort:
          shutil.rmtree(self._task_storage_path)
        else:
          os.rmdir(self._task_storage_path)

      self._merge_task_storage_path = None
      self._processed_task_storage_path = None
      self._task_storage_path = None

      self._processing_configuration.task_storage_path = None



#!/usr/bin/env python3
# -*_ coding: utf-8 -*-
"""Tests for the dpkg.log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import dpkg

from tests.parsers.text_plugins import test_lib


class DpkgTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the dpkg log text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = dpkg.DpkgTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['dpkg.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2009-02-25T11:45:23',
        'body': 'conffile /etc/X11/Xsession keep',
        'data_type': 'linux:dpkg_log:entry'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""JSON-L parser plugin for Google Cloud (GCP) log files."""

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.parsers import jsonl_parser
from plaso.parsers.jsonl_plugins import interface


class GCPLogEventData(events.EventData):
  """Google Cloud (GCP) log event data.

  Attributes:
    container (str): TODO
    event_subtype (str): JSON event sub type or protocol buffer method.
    event_type (str): TODO
    filename (str): TODO
    firewall_rules (list[str]): firewall rules.
    firewall_source_ranges (list[str]): firewall source ranges.
    log_name (str): name of the log entry.
    message (str): TODO
    policy_deltas (list[str]): TODO
    recorded_time (dfdatetime.DateTimeValues): date and time the log entry
        was recorded.
    request_account_identifier (str): GCP account identifier of the request.
    request_description (str): description of the request.
    request_direction (str): direction of the request.
    request_email (str): email address of the request.
    request_member (str): member of the request.
    request_metadata (list[str]): request metadata values.
    request_name (str): name of the request.
    request_target_tags (str): TODO
    resource_labels (list[str]): resource labels.
    resource_name (str): name of the resource.
    service_account_display_name (str): display name of the service account.
    service_name (str): name of the servie.
    severity (str): log entry severity.
    text_payload (str): text payload for logs not using a JSON or proto payload.
    user (str): user principal performing the logged action.
  """

  DATA_TYPE = 'gcp:log:entry'

  def __init__(self):
    """Initializes event data."""
    super(GCPLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.container = None
    self.event_subtype = None
    self.event_type = None
    self.filename = None
    self.firewall_rules = None
    self.firewall_source_ranges = None
    self.log_name = None
    self.message = None
    self.policy_deltas = None
    self.recorded_time = None
    self.request_account_identifier = None
    self.request_description = None
    self.request_direction = None
    self.request_email = None
    self.request_member = None
    self.request_metadata = None
    self.request_name = None
    self.request_target_tags = None
    self.resource_labels = None
    self.resource_name = None
    self.service_account_display_name = None
    self.service_name = None
    self.severity = None
    self.text_payload = None
    self.user = None


class GCPLogJSONLPlugin(interface.JSONLPlugin):
  """JSON-L parser plugin for Google Cloud (GCP) log files."""

  NAME = 'gcp_log'
  DATA_FORMAT = 'Google Cloud (GCP) log'

  def _ParseJSONPayload(self, json_dict, event_data):
    """Extracts information from a jsonPayload value.

    Args:
      json_dict (dict): JSON dictionary of the log record.
      event_data (GCPLogEventData): event data.
    """
    json_payload = self._GetJSONValue(json_dict, 'jsonPayload')
    if not json_payload:
      return

    event_data.container = self._GetJSONValue(json_payload, 'container')
    event_data.event_subtype = self._GetJSONValue(json_payload, 'event_subtype')
    event_data.event_type = self._GetJSONValue(json_payload, 'event_type')
    event_data.filename = self._GetJSONValue(json_payload, 'filename')
    event_data.message = self._GetJSONValue(json_payload, 'message')

    actor_json = self._GetJSONValue(json_payload, 'actor')
    if actor_json:
      event_data.user = self._GetJSONValue(actor_json, 'user')

  def _ParseProtoPayload(self, json_dict, event_data):
    """Extracts information from a protoPayload value.

    Args:
      json_dict (dict): JSON dictionary of the log record.
      event_data (GCPLogEventData): event data.
    """
    proto_payload = self._GetJSONValue(json_dict, 'protoPayload')
    if not proto_payload:
      return

    authentication_info = self._GetJSONValue(
        proto_payload, 'authenticationInfo')
    if authentication_info and not event_data.user:
      event_data.user = self._GetJSONValue(
          authentication_info, 'principalEmail')

    request_metadata = self._GetJSONValue(
        proto_payload, 'requestMetadata', default_value={})
    event_data.request_metadata = [
        '{0:s}: {1!s}'.format(name, value)
        for name, value in request_metadata.items()]

    event_data.service_name = self._GetJSONValue(proto_payload, 'serviceName')
    event_data.resource_name = self._GetJSONValue(proto_payload, 'resourceName')

    method_name = self._GetJSONValue(proto_payload, 'methodName')
    if method_name and not event_data.event_subtype:
      event_data.event_subtype = method_name

    self._ParseProtoPayloadRequest(proto_payload, event_data)
    self._ParseProtoPayloadServiceData(proto_payload, event_data)

  def _ParseProtoPayloadRequest(self, json_dict, event_data):
    """Extracts information from the request field of a protoPayload field.

    Args:
      json_dict (dict): JSON dictionary of the protoPayload value.
      event_data (GCPLogEventData): event data.
    """
    request = self._GetJSONValue(json_dict, 'request')
    if not request:
      return

    event_data.request_account_identifier = self._GetJSONValue(
        request, 'account_id')
    event_data.request_description = self._GetJSONValue(request, 'description')
    event_data.request_direction = self._GetJSONValue(request, 'direction')
    event_data.request_email = self._GetJSONValue(request, 'email')
    event_data.request_member = self._GetJSONValue(request, 'member')
    event_data.request_name = self._GetJSONValue(request, 'name')
    event_data.request_target_tags = self._GetJSONValue(request, 'targetTags')

    # Firewall specific attributes.
    event_data.firewall_source_ranges = self._GetJSONValue(
        request, 'sourceRanges')

    firewall_rules = []

    alloweds = self._GetJSONValue(request, 'alloweds', default_value=[])
    for allowed in alloweds:
      ip_protocol = self._GetJSONValue(allowed, 'IPProtocol')
      ports = self._GetJSONValue(allowed, 'ports', default_value='all')

      firewall_rule = 'ALLOW: {0:s} {1!s}'.format(ip_protocol, ports)
      firewall_rules.append(firewall_rule)

    denieds = self._GetJSONValue(request, 'denieds', default_value=[])
    for denied in denieds:
      ip_protocol = self._GetJSONValue(denied, 'IPProtocol')
      ports = self._GetJSONValue(denied, 'ports', default_value='all')

      firewall_rule = 'DENY: {0:s} {1!s}'.format(ip_protocol, ports)
      firewall_rules.append(firewall_rule)

    event_data.firewall_rules = firewall_rules or None

    # Service account specific attributes
    service_account = self._GetJSONValue(request, 'service_account')
    if service_account:
      event_data.service_account_display_name = self._GetJSONValue(
          service_account, 'display_name')

  def _ParseProtoPayloadServiceData(self, json_dict, event_data):
    """Extracts information from the serviceData in the protoPayload value.

    Args:
      json_dict (dict): JSON dictionary of the protoPayload value.
      event_data (GCPLogEventData): event data.
    """
    service_data = self._GetJSONValue(json_dict, 'serviceData')
    if not service_data:
      return

    policy_delta = self._GetJSONValue(service_data, 'policyDelta')
    if not policy_delta:
      return

    policy_deltas = []

    binding_deltas = self._GetJSONValue(
        policy_delta, 'bindingDeltas', default_value=[])
    for binding_delta_value in binding_deltas:
      action = self._GetJSONValue(
          binding_delta_value, 'action', default_value='')
      member = self._GetJSONValue(
          binding_delta_value, 'member', default_value='')
      role = self._GetJSONValue(binding_delta_value, 'role', default_value='')

      policy_delta = '{0:s} {1:s} with role {2:s}'.format(action, member, role)
      policy_deltas.append(policy_delta)

    event_data.policy_deltas = policy_deltas or None

  def _ParseRecord(self, parser_mediator, json_dict):
    """Parses a Google Cloud (GCP) log record.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      json_dict (dict): JSON dictionary of the log record.
    """
    resource = self._GetJSONValue(json_dict, 'resource', default_value={})
    labels = self._GetJSONValue(resource, 'labels', default_value={})

    resource_labels = [
        '{0:s}: {1!s}'.format(name, value) for name, value in labels.items()]

    event_data = GCPLogEventData()
    event_data.log_name = self._GetJSONValue(json_dict, 'logName')
    event_data.recorded_time = self._ParseISO8601DateTimeString(
        parser_mediator, json_dict, 'timestamp')
    event_data.resource_labels = resource_labels or None
    event_data.severity = self._GetJSONValue(json_dict, 'severity')
    event_data.text_payload = self._GetJSONValue(json_dict, 'textPayload')

    self._ParseJSONPayload(json_dict, event_data)
    self._ParseProtoPayload(json_dict, event_data)

    parser_mediator.ProduceEventData(event_data)

  def CheckRequiredFormat(self, json_dict):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      json_dict (dict): JSON dictionary of the log record.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    log_name = json_dict.get('logName') or None
    timestamp = json_dict.get('timestamp') or None

    if None in (log_name, timestamp):
      return False

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()

    try:
      date_time.CopyFromStringISO8601(timestamp)
    except ValueError:
      return False

    return True


jsonl_parser.JSONLParser.RegisterPlugin(GCPLogJSONLPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome cookie database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import chrome_cookies

from tests.parsers.sqlite_plugins import test_lib


class Chrome17CookiesPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Chrome 17-65 cookie database plugin."""

  def testProcess(self):
    """Tests the Process function on a Chrome cookie database file."""
    plugin = chrome_cookies.Chrome17CookiePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(['cookies.db'], plugin)

    # 560 Chrome cookie and 43 cookie plugin.
    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 603)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': '2011-08-25T21:50:27.292367+00:00',
        'creation_time': '2011-08-25T21:48:20.792703+00:00',
        'cookie_name': 'leo_auth_token',
        'data': (
            '"LIM:137381921:a:21600:1314308846:'
            '8797616454cd88b46baad44abb3c29ac45e467d7"'),
        'data_type': 'chrome:cookie:entry',
        'expiration_time': '2011-11-23T21:48:19.792703+00:00',
        'host': 'www.linkedin.com',
        'httponly': False,
        'persistent': True,
        'url': 'http://www.linkedin.com/'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 45)
    self.CheckEventData(event_data, expected_event_values)


class Chrome66CookiesPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Chrome 66 Cookies database plugin."""

  def testProcess(self):
    """Tests the Process function on a Chrome cookie database file."""
    plugin = chrome_cookies.Chrome66CookiePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['Cookies-68.0.3440.106'], plugin)

    # 5 Chrome cookie and 1 cookie plugin.
    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': '2018-08-14T15:03:45.489599+00:00',
        'cookie_name': '__utma',
        'creation_time': '2018-08-14T15:03:43.650324+00:00',
        'data': '',
        'data_type': 'chrome:cookie:entry',
        'expiration_time': '2020-08-13T15:03:45.000000+00:00',
        'host': 'google.com',
        'httponly': False,
        'persistent': True,
        'url': 'http://google.com/gmail/about/'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'cookie_name': '__utma',
        'data_type': 'cookie:google:analytics:utma',
        'url': 'http://google.com/gmail/about/'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The single process extraction engine."""

import collections
import os
import pdb
import threading
import time

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.containers import counts
from plaso.containers import event_sources
from plaso.containers import events
from plaso.engine import engine
from plaso.engine import extractors
from plaso.engine import logger
from plaso.engine import process_info
from plaso.engine import timeliner
from plaso.engine import worker
from plaso.lib import definitions
from plaso.lib import errors
from plaso.parsers import mediator as parsers_mediator


class SingleProcessEngine(engine.BaseEngine):
  """Single process extraction engine."""

  _CONTAINER_TYPE_EVENT_DATA_STREAM = events.EventDataStream.CONTAINER_TYPE

  # Maximum number of dfVFS file system objects to cache.
  _FILE_SYSTEM_CACHE_SIZE = 3

  def __init__(self, status_update_callback=None):
    """Initializes a single process extraction engine.

    Args:
      status_update_callback (Optional[function]): callback function for status
          updates.
    """
    super(SingleProcessEngine, self).__init__()
    self._current_display_name = ''
    self._event_data_timeliner = None
    self._extraction_worker = None
    self._file_system_cache = []
    self._number_of_consumed_event_data = 0
    self._number_of_consumed_sources = 0
    self._number_of_produced_events = 0
    self._parser_mediator = None
    self._parsers_counter = None
    self._path_spec_extractor = extractors.PathSpecExtractor()
    self._pid = os.getpid()
    self._process_information = process_info.ProcessInfo(self._pid)
    self._processing_configuration = None
    self._resolver_context = None
    self._status = definitions.STATUS_INDICATOR_IDLE
    self._status_update_active = False
    self._status_update_callback = status_update_callback
    self._status_update_thread = None
    self._storage_writer = None

  def _CacheFileSystem(self, file_system):
    """Caches a dfVFS file system object.

    Keeping and additional reference to a dfVFS file system object causes the
    object to remain cached in the resolver context. This minimizes the number
    times the file system is re-opened.

    Args:
      file_system (dfvfs.FileSystem): file system.
    """
    if file_system not in self._file_system_cache:
      if len(self._file_system_cache) == self._FILE_SYSTEM_CACHE_SIZE:
        self._file_system_cache.pop(0)
      self._file_system_cache.append(file_system)

    elif len(self._file_system_cache) == self._FILE_SYSTEM_CACHE_SIZE:
      # Move the file system to the end of the list to preserve the most
      # recently file system object.
      self._file_system_cache.remove(file_system)
      self._file_system_cache.append(file_system)

  def _CheckExcludedPathSpec(self, file_system, path_spec):
    """Determines if the path specification should be excluded from extraction.

    Args:
      file_system (dfvfs.FileSystem): file system which the path specification
          is part of.
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      bool: True if the path specification should be excluded from extraction.
    """
    for find_spec in self._excluded_file_system_find_specs or []:
      if find_spec.ComparePathSpecLocation(path_spec, file_system):
        return True

    return False

  def _CollectInitialEventSources(
      self, parser_mediator, file_system_path_specs):
    """Collects the initial event sources.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      file_system_path_specs (list[dfvfs.PathSpec]): path specifications of
          the source file systems to process.
    """
    self._status = definitions.STATUS_INDICATOR_COLLECTING

    included_find_specs = self.GetCollectionIncludedFindSpecs()

    for file_system_path_spec in file_system_path_specs:
      if self._abort:
        break

      file_system = path_spec_resolver.Resolver.OpenFileSystem(
          file_system_path_spec, resolver_context=self._resolver_context)

      path_spec_generator = self._path_spec_extractor.ExtractPathSpecs(
          file_system_path_spec, find_specs=included_find_specs,
          recurse_file_system=False, resolver_context=self._resolver_context)
      for path_spec in path_spec_generator:
        if self._abort:
          break

        if self._CheckExcludedPathSpec(file_system, path_spec):
          display_name = parser_mediator.GetDisplayNameForPathSpec(path_spec)
          logger.debug('Excluded from extraction: {0:s}.'.format(display_name))
          continue

        # TODO: determine if event sources should be DataStream or FileEntry
        # or both.
        event_source = event_sources.FileEntryEventSource(path_spec=path_spec)
        parser_mediator.ProduceEventSource(event_source)

  def _ProcessEventData(self):
    """Generate events from event data."""
    if self._processing_profiler:
      self._processing_profiler.StartTiming('process_event_data')

    self._status = definitions.STATUS_INDICATOR_TIMELINING

    if self._processing_profiler:
      self._processing_profiler.StartTiming('get_event_data')

    event_data = self._storage_writer.GetFirstWrittenEventData()

    if self._processing_profiler:
      self._processing_profiler.StopTiming('get_event_data')

    while event_data:
      if self._abort:
        break

      event_data_stream_identifier = event_data.GetEventDataStreamIdentifier()

      event_data_stream = None
      if event_data_stream_identifier:
        if self._processing_profiler:
          self._processing_profiler.StartTiming('get_event_data_stream')

        event_data_stream = (
            self._storage_writer.GetAttributeContainerByIdentifier(
                self._CONTAINER_TYPE_EVENT_DATA_STREAM,
                event_data_stream_identifier))

        if self._processing_profiler:
          self._processing_profiler.StopTiming('get_event_data_stream')

      self._event_data_timeliner.ProcessEventData(
          self._storage_writer, event_data, event_data_stream)

      self._number_of_consumed_event_data += 1
      self._number_of_produced_events += (
          self._event_data_timeliner.number_of_produced_events)

      # TODO: track number of consumed event data containers?

      if self._processing_profiler:
        self._processing_profiler.StartTiming('get_event_data')

      event_data = self._storage_writer.GetNextWrittenEventData()

      if self._processing_profiler:
        self._processing_profiler.StopTiming('get_event_data')

    if self._abort:
      self._status = definitions.STATUS_INDICATOR_ABORTED
    else:
      self._status = definitions.STATUS_INDICATOR_COMPLETED

    if self._processing_profiler:
      self._processing_profiler.StopTiming('process_event_data')

  def _ProcessEventSources(self, storage_writer, parser_mediator):
    """Processes event sources.

    Args:
      storage_writer (StorageWriter): storage writer for a session storage.
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
    """
    self._status = definitions.STATUS_INDICATOR_RUNNING

    if self._processing_profiler:
      self._processing_profiler.StartTiming('get_event_source')

    event_source = storage_writer.GetFirstWrittenEventSource()

    if self._processing_profiler:
      self._processing_profiler.StopTiming('get_event_source')

    while event_source:
      if self._abort:
        break

      self._ProcessPathSpec(parser_mediator, event_source.path_spec)

      self._number_of_consumed_sources += 1

      if self._processing_profiler:
        self._processing_profiler.StartTiming('get_event_source')

      event_source = storage_writer.GetNextWrittenEventSource()

      if self._processing_profiler:
        self._processing_profiler.StopTiming('get_event_source')

  def _ProcessPathSpec(self, parser_mediator, path_spec):
    """Processes a path specification.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      path_spec (dfvfs.PathSpec): path specification.
    """
    try:
      self._current_display_name = parser_mediator.GetDisplayNameForPathSpec(
          path_spec)

      file_entry = path_spec_resolver.Resolver.OpenFileEntry(
          path_spec, resolver_context=parser_mediator.resolver_context)
      if file_entry is None:
        logger.warning('Unable to open file entry: {0:s}'.format(
            self._current_display_name))
        return

      file_system = file_entry.GetFileSystem()

      if (path_spec and not path_spec.IsSystemLevel() and
          path_spec.type_indicator != dfvfs_definitions.TYPE_INDICATOR_GZIP):
        self._CacheFileSystem(file_system)

      if self._CheckExcludedPathSpec(file_system, path_spec):
        logger.debug('Excluded from extraction: {0:s}.'.format(
            self._current_display_name))
        return

      self._extraction_worker.ProcessFileEntry(parser_mediator, file_entry)

    except KeyboardInterrupt:
      self._abort = True

      self._processing_status.aborted = True
      if self._status_update_callback:
        self._status_update_callback(self._processing_status)

    # All exceptions need to be caught here to prevent the worker
    # from being killed by an uncaught exception.
    except Exception as exception:  # pylint: disable=broad-except
      parser_mediator.ProduceExtractionWarning((
          'unable to process path specification with error: '
          '{0!s}').format(exception), path_spec=path_spec)

      if getattr(self._processing_configuration, 'debug_output', False):
        self._StopStatusUpdateThread()

        logger.warning(
            'Unhandled exception while processing path spec: {0:s}.'.format(
                self._current_display_name))
        logger.exception(exception)

        pdb.post_mortem()

        self._StartStatusUpdateThread()

  def _ProcessSource(self, parser_mediator, file_system_path_specs):
    """Processes file systems within a source.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      file_system_path_specs (list[dfvfs.PathSpec]): path specifications of
          the source file systems to process.
    """
    self._current_display_name = ''
    self._number_of_consumed_sources = 0

    if self._processing_profiler:
      self._processing_profiler.StartTiming('process_source')

    self._CollectInitialEventSources(
        parser_mediator, file_system_path_specs)

    self._ProcessEventSources(self._storage_writer, parser_mediator)

    if self._processing_profiler:
      self._processing_profiler.StopTiming('process_source')

    if self._abort:
      self._status = definitions.STATUS_INDICATOR_ABORTED
    else:
      self._status = definitions.STATUS_INDICATOR_COMPLETED

  def _StartStatusUpdateThread(self):
    """Starts the status update thread."""
    self._status_update_active = True
    self._status_update_thread = threading.Thread(
        name='Status update', target=self._StatusUpdateThreadMain)
    self._status_update_thread.start()

  def _StatusUpdateThreadMain(self):
    """Main function of the status update thread."""
    while self._status_update_active:
      self._UpdateStatus()

      time.sleep(self._status_update_interval)

  def _StopStatusUpdateThread(self):
    """Stops the status update thread."""
    if self._status_update_thread:
      self._status_update_active = False
      if self._status_update_thread.is_alive():
        self._status_update_thread.join()
      self._status_update_thread = None

  def _UpdateStatus(self):
    """Updates the processing status."""
    status = self._extraction_worker.processing_status
    if status == definitions.STATUS_INDICATOR_IDLE:
      status = self._status
    if status == definitions.STATUS_INDICATOR_IDLE:
      status = definitions.STATUS_INDICATOR_RUNNING

    used_memory = self._process_information.GetUsedMemory() or 0

    self._processing_status.UpdateForemanStatus(
        self._name, status, self._pid, used_memory, self._current_display_name,
        self._number_of_consumed_sources,
        self._parser_mediator.number_of_produced_event_sources,
        self._number_of_consumed_event_data,
        self._parser_mediator.number_of_produced_event_data,
        0, self._number_of_produced_events,
        0, 0,
        0, 0)

    if self._status_update_callback:
      self._status_update_callback(self._processing_status)

  def _CreateParserMediator(
      self, storage_writer, resolver_context, processing_configuration,
      system_configurations):
    """Creates a parser mediator.

    Args:
      storage_writer (StorageWriter): storage writer for a session storage.
      resolver_context (dfvfs.Context): resolver context.
      processing_configuration (ProcessingConfiguration): processing
          configuration.
      system_configurations (list[SystemConfigurationArtifact]): system
          configurations.

    Returns:
      ParserMediator: parser mediator.

    Raises:
      BadConfigOption: if an invalid collection filter was specified.
    """
    # TODO: get environment_variables per system_configuration
    environment_variables = None
    if self.knowledge_base:
      environment_variables = self.knowledge_base.GetEnvironmentVariables()

    user_accounts = list(storage_writer.GetAttributeContainers('user_account'))

    try:
      self.BuildCollectionFilters(
          environment_variables, user_accounts,
          artifact_filter_names=processing_configuration.artifact_filters,
          filter_file_path=processing_configuration.filter_file)
    except errors.InvalidFilter as exception:
      raise errors.BadConfigOption(
          'Unable to build collection filters with error: {0!s}'.format(
              exception))

    parser_mediator = parsers_mediator.ParserMediator(
        registry_find_specs=self._registry_find_specs,
        resolver_context=resolver_context,
        system_configurations=system_configurations)

    parser_mediator.SetExtractWinEvtResources(
        processing_configuration.extraction.extract_winevt_resources)
    parser_mediator.SetPreferredCodepage(
        processing_configuration.preferred_codepage)
    parser_mediator.SetPreferredLanguage(
        processing_configuration.preferred_language)
    parser_mediator.SetTemporaryDirectory(
        processing_configuration.temporary_directory)

    return parser_mediator

  def ProcessSource(
      self, storage_writer, resolver_context, processing_configuration,
      system_configurations, file_system_path_specs):
    """Processes file systems within a source.

    Args:
      storage_writer (StorageWriter): storage writer for a session storage.
      resolver_context (dfvfs.Context): resolver context.
      processing_configuration (ProcessingConfiguration): processing
          configuration.
      system_configurations (list[SystemConfigurationArtifact]): system
          configurations.
      file_system_path_specs (list[dfvfs.PathSpec]): path specifications of
          the source file systems to process.

    Returns:
      ProcessingStatus: processing status.

    Raises:
      BadConfigOption: if an invalid collection filter was specified or if
          the preferred time zone is invalid.
    """
    if not self._artifacts_registry:
      # TODO: refactor.
      self.BuildArtifactsRegistry(
          processing_configuration.artifact_definitions_path,
          processing_configuration.custom_artifacts_path)

    parser_mediator = self._CreateParserMediator(
        storage_writer, resolver_context, processing_configuration,
        system_configurations)
    parser_mediator.SetStorageWriter(storage_writer)

    self._extraction_worker = worker.EventExtractionWorker(
        force_parser=processing_configuration.force_parser,
        parser_filter_expression=(
            processing_configuration.parser_filter_expression))

    self._extraction_worker.SetExtractionConfiguration(
        processing_configuration.extraction)

    self._event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=processing_configuration.data_location,
        preferred_year=processing_configuration.preferred_year,
        system_configurations=system_configurations)

    try:
      self._event_data_timeliner.SetPreferredTimeZone(
          processing_configuration.preferred_time_zone)
    except ValueError as exception:
      raise errors.BadConfigOption(exception)

    self._parser_mediator = parser_mediator
    self._processing_configuration = processing_configuration
    self._resolver_context = resolver_context
    self._storage_writer = storage_writer

    logger.debug('Processing started.')

    parser_mediator.StartProfiling(
        self._processing_configuration.profiling, self._name,
        self._process_information)
    self._StartProfiling(self._processing_configuration.profiling)

    if self._analyzers_profiler:
      self._extraction_worker.SetAnalyzersProfiler(self._analyzers_profiler)

    if self._processing_profiler:
      self._extraction_worker.SetProcessingProfiler(self._processing_profiler)

    if self._serializers_profiler:
      self._storage_writer.SetSerializersProfiler(self._serializers_profiler)

    if self._storage_profiler:
      self._storage_writer.SetStorageProfiler(self._storage_profiler)

    self._StartStatusUpdateThread()

    self._parsers_counter = collections.Counter({
        parser_count.name: parser_count
        for parser_count in self._storage_writer.GetAttributeContainers(
            'parser_count')})

    try:
      self._ProcessSource(parser_mediator, file_system_path_specs)

      self._ProcessEventData()

    finally:
      # Stop the status update thread after close of the storage writer
      # so we include the storage sync to disk in the status updates.
      self._StopStatusUpdateThread()

      if self._analyzers_profiler:
        self._extraction_worker.SetAnalyzersProfiler(None)

      if self._processing_profiler:
        self._extraction_worker.SetProcessingProfiler(None)

      if self._serializers_profiler:
        self._storage_writer.SetSerializersProfiler(None)

      if self._storage_profiler:
        self._storage_writer.SetStorageProfiler(None)

      self._StopProfiling()
      parser_mediator.StopProfiling()

    for key, value in self._event_data_timeliner.parsers_counter.items():
      parser_count = self._parsers_counter.get(key, None)
      if parser_count:
        parser_count.number_of_events += value
        self._storage_writer.UpdateAttributeContainer(parser_count)
      else:
        parser_count = counts.ParserCount(name=key, number_of_events=value)
        self._parsers_counter[key] = parser_count
        self._storage_writer.AddAttributeContainer(parser_count)

    # TODO: remove after completion event and event data split.
    for key, value in parser_mediator.parsers_counter.items():
      parser_count = self._parsers_counter.get(key, None)
      if parser_count:
        parser_count.number_of_events += value
        self._storage_writer.UpdateAttributeContainer(parser_count)
      else:
        parser_count = counts.ParserCount(name=key, number_of_events=value)
        self._parsers_counter[key] = parser_count
        self._storage_writer.AddAttributeContainer(parser_count)

    if self._abort:
      logger.debug('Processing aborted.')
      self._processing_status.aborted = True
    else:
      logger.debug('Processing completed.')

    # Update the status view one last time.
    self._UpdateStatus()

    self._event_data_timeliner = None
    self._extraction_worker = None
    self._file_system_cache = []
    self._parser_mediator = None
    self._processing_configuration = None
    self._resolver_context = None
    self._storage_writer = None

    return self._processing_status



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Drive database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import gdrive

from tests.parsers.sqlite_plugins import test_lib


class GoogleDrivePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Drive database plugin."""

  def testProcess(self):
    """Tests the Process function on a Google Drive database file."""
    plugin = gdrive.GoogleDrivePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(['snapshot.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 20)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test a snapshot local entry.
    expected_event_values = {
        'data_type': 'gdrive:snapshot:local_entry',
        'modification_time': '2014-01-28T00:11:25+00:00',
        'path': (
            '%local_sync_root%/Top Secret/Enn meiri '
            'leyndarmál/Sýnileiki - Örverpi.gdoc'),
        'size': 184}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 15)
    self.CheckEventData(event_data, expected_event_values)

    # Test a snapshot cloud entry.
    expected_event_values = {
        'data_type': 'gdrive:snapshot:cloud_entry',
        'creation_time': '2014-01-28T00:12:26+00:00',
        'document_type': 6,
        'modification_time': '2014-01-28T00:12:27+00:00',
        'path': '/Almenningur/Saklausa hliðin',
        'size': 0,
        'url': (
            'https://docs.google.com/document/d/1ypXwXhQWliiMSQN9S5M0K6Wh39XF4U'
            'z4GmY-njMf-Z0/edit?usp=docslist_api')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 8)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite parser plugin for MacOS TCC database files."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class MacOSTCCEntry(events.EventData):
  """MacOS TCC event data.

  Attributes:
    allowed (bool): whether access to the service was allowed.
    client (str): name of the client requesting access to the service.
    modification_time (dfdatetime.DateTimeValues): date and time of the entry
        last modification.
    prompt_count (int): number of times an application prompted the user for
        access to a service.
    query (str): SQL query that was used to obtain the event data.
    service (str): name of the service.
  """

  DATA_TYPE = 'macos:tcc_entry'

  def __init__(self):
    """Initializes event data."""
    super(MacOSTCCEntry, self).__init__(data_type=self.DATA_TYPE)
    self.allowed = None
    self.client = None
    self.modification_time = None
    self.prompt_count = None
    self.query = None
    self.service = None


class MacOSTCCPlugin(interface.SQLitePlugin):
  """SQLite parser plugin for MacOS TCC database files.

  The MacOS Transparency, Consent, Control (TCC) database file is typically
  stored in:
  /Library/Application Support/com.apple.TCC/TCC.db
  /Users/<username>/Library/Application Support/com.apple.TCC/TCC.db
  """

  NAME = 'macostcc'
  DATA_FORMAT = (
      'MacOS Transparency, Consent, Control (TCC) SQLite database (TCC.db) '
      'file')

  REQUIRED_STRUCTURE = {
      'access': frozenset([
          'allowed', 'client', 'last_modified', 'prompt_count', 'service']),
      'access_overrides': frozenset([]),
      'active_policy': frozenset([]),
      'admin': frozenset([]),
      'expired': frozenset([]),
      'policies': frozenset([])}

  QUERIES = [(
      ('SELECT service, client, allowed, prompt_count, last_modified '
       'FROM access;'), 'ParseTCCEntry')]

  SCHEMAS = [{
      'access': (
          'CREATE TABLE access ( service TEXT NOT NULL, client TEXT NOT NULL, '
          'client_type INTEGER NOT NULL, allowed INTEGER NOT NULL, '
          'prompt_count INTEGER NOT NULL, csreq BLOB, policy_id INTEGER, '
          'indirect_object_identifier_type INTEGER, '
          'indirect_object_identifier TEXT, indirect_object_code_identity '
          'BLOB, flags INTEGER, last_modified INTEGER NOT NULL DEFAULT '
          '(CAST(strftime(\'%s\',\'now\') AS INTEGER)), PRIMARY KEY (service, '
          'client, client_type, indirect_object_identifier), FOREIGN KEY '
          '(policy_id) REFERENCES policies(id) ON DELETE CASCADE ON UPDATE '
          'CASCADE)'),
      'access_overrides': (
          'CREATE TABLE access_overrides ( service TEXT NOT NULL PRIMARY KEY)'),
      'active_policy': (
          'CREATE TABLE active_policy ( client TEXT NOT NULL, client_type '
          'INTEGER NOT NULL, policy_id INTEGER NOT NULL, PRIMARY KEY (client, '
          'client_type), FOREIGN KEY (policy_id) REFERENCES policies(id) ON '
          'DELETE CASCADE ON UPDATE CASCADE)'),
      'admin': (
          'CREATE TABLE admin (key TEXT PRIMARY KEY NOT NULL, value INTEGER '
          'NOT NULL)'),
      'expired': (
          'CREATE TABLE expired ( service TEXT NOT NULL, client TEXT NOT '
          'NULL, client_type INTEGER NOT NULL, csreq BLOB, last_modified '
          'INTEGER NOT NULL , expired_at INTEGER NOT NULL DEFAULT '
          '(CAST(strftime(\'%s\',\'now\') AS INTEGER)), PRIMARY KEY (service, '
          'client, client_type))'),
      'policies': (
          'CREATE TABLE policies ( id INTEGER NOT NULL PRIMARY KEY, bundle_id '
          'TEXT NOT NULL, uuid TEXT NOT NULL, display TEXT NOT NULL, UNIQUE '
          '(bundle_id, uuid))')}]

  def _GetDateTimeRowValue(self, query_hash, row, value_name):
    """Retrieves a date and time value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.PosixTime: date and time value or None if not available.
    """
    timestamp = self._GetRowValue(query_hash, row, value_name)
    if timestamp is None:
      return None

    return dfdatetime_posix_time.PosixTime(timestamp=timestamp)

  def ParseTCCEntry(self, parser_mediator, query, row, **unused_kwargs):
    """Parses an application usage row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = MacOSTCCEntry()
    event_data.allowed = self._GetRowValue(query_hash, row, 'allowed')
    event_data.client = self._GetRowValue(query_hash, row, 'client')
    event_data.modification_time = self._GetDateTimeRowValue(
        query_hash, row, 'last_modified')
    event_data.prompt_count = self._GetRowValue(query_hash, row, 'prompt_count')
    event_data.query = query
    event_data.service = self._GetRowValue(query_hash, row, 'service')

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(MacOSTCCPlugin)



# -*- coding: utf-8 -*-
"""Plist parser plugin for MacOS Bluetooth plist files.

Fields within the plist key: com.apple.bluetooth.plist

LastInquiryUpdate:
  Device connected via Bluetooth discovery. Updated when a device is detected
  in discovery mode. E.g. Bluetooth headphone power on. Pairing is not required
  for a device to be discovered and cached.

LastNameUpdate:
  When the human name was last set. Usually done only once during initial setup.

LastServicesUpdate:
  Time set when device was polled to determine what it is. Usually done at setup
  or manually requested via advanced menu.
"""

from plaso.containers import events
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


class MacOSBluetoothEventData(events.EventData):
  """MacOS Bluetooth event data.

  Attributes:
    device_identifier (str): identifier of the device.
    device_name (str): name of the device.
    inquiry_time (dfdatetime.DateTimeValues): date and time of the most
        recent inquiry (connection during discovery mode) of a Bluetooth device.
    is_paired (bool): True if the device has been paired.
    name_update_time (dfdatetime.DateTimeValues): date and time of the most
        recent update of the human name.
    services_update_time (dfdatetime.DateTimeValues): date and time of the most
        recent poll of a Bluetooth device.
  """

  DATA_TYPE = 'macos:bluetooth:entry'

  def __init__(self):
    """Initializes event data."""
    super(MacOSBluetoothEventData, self).__init__(data_type=self.DATA_TYPE)
    self.device_identifier = None
    self.device_name = None
    self.inquiry_time = None
    self.is_paired = None
    self.name_update_time = None
    self.services_update_time = None


class MacOSBluetoothPlistPlugin(interface.PlistPlugin):
  """Plist parser plugin for MacOS Bluetooth plist files."""

  NAME = 'macos_bluetooth'
  DATA_FORMAT = 'MacOS Bluetooth plist file'

  PLIST_PATH_FILTERS = frozenset([
      interface.PlistPathFilter('com.apple.bluetooth.plist')])

  PLIST_KEYS = frozenset(['DeviceCache', 'PairedDevices'])

  # pylint: disable=arguments-differ
  def _ParsePlist(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant BT entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    paired_devices = match.get('PairedDevices', [])

    for device_identifier, plist_key in match.get('DeviceCache', {}).items():
      event_data = MacOSBluetoothEventData()
      event_data.device_identifier = device_identifier
      event_data.device_name = plist_key.get('Name', None)
      event_data.inquiry_time = self._GetDateTimeValueFromPlistKey(
          plist_key, 'LastInquiryUpdate')
      event_data.is_paired = device_identifier in paired_devices
      event_data.name_update_time = self._GetDateTimeValueFromPlistKey(
          plist_key, 'LastNameUpdate')
      event_data.services_update_time = self._GetDateTimeValueFromPlistKey(
          plist_key, 'LastServicesUpdate')

      parser_mediator.ProduceEventData(event_data)


plist.PlistParser.RegisterPlugin(MacOSBluetoothPlistPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows languages helper."""

import unittest

from plaso.helpers.windows import languages

from tests import test_lib as shared_test_lib


class WindowsLanguageHelperTest(shared_test_lib.BaseTestCase):
  """Tests for the Windows languages helper."""

  def testGetLanguageTagForLCID(self):
    """Tests the GetLanguageTagForLCID function."""
    language_tag = languages.WindowsLanguageHelper.GetLanguageTagForLCID(0x040f)
    self.assertEqual(language_tag, 'is-IS')

    language_tag = languages.WindowsLanguageHelper.GetLanguageTagForLCID(0xffff)
    self.assertIsNone(language_tag)

  def testGetLCIDForLanguageTag(self):
    """Tests the GetLCIDForLanguageTag function."""
    lcid = languages.WindowsLanguageHelper.GetLCIDForLanguageTag('is-IS')
    self.assertEqual(lcid, 0x040f)

    lcid = languages.WindowsLanguageHelper.GetLCIDForLanguageTag('bogus')
    self.assertIsNone(lcid)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Apache access log text parser plugin."""

import unittest

from plaso.containers import warnings
from plaso.parsers.text_plugins import apache_access

from tests.parsers.text_plugins import test_lib


class ApacheAccessLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for Apache access log text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = apache_access.ApacheAccessLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['apache_access.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 14)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test combined log format event.
    # 13/Jan/2016:19:31:20 +0200
    expected_event_values = {
        'data_type': 'apache:access_log:entry',
        'http_request': (
            'GET /wp-content/themes/darkmode/evil.php?cmd=uname+-a HTTP/1.1'),
        'http_request_referer': 'http://localhost/',
        'http_request_user_agent': (
            'Mozilla/5.0 (X11; Linux i686; rv:2.0b12pre) Gecko/20100101 '
            'Firefox/4'),
        'http_response_code': 200,
        'http_response_bytes': 694,
        'ip_address': '192.168.0.2',
        'recorded_time': '2016-01-13T19:31:20+02:00',
        'remote_name': None,
        'user_name': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 4)
    self.CheckEventData(event_data, expected_event_values)

    # Test common log format parser event.
    expected_event_values = {
        'data_type': 'apache:access_log:entry',
        'http_request': (
            'GET /wp-content/themes/darkmode/header.php?install2 HTTP/1.1'),
        'http_response_code': 200,
        'http_response_bytes': 494,
        'ip_address': '10.0.0.1',
        'recorded_time': '2016-01-13T19:31:16+00:00',
        'remote_name': None,
        'user_name': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Test vhost_combined log format event.
    expected_event_values = {
        'data_type': 'apache:access_log:entry',
        'http_request': 'GET /wp-content/themes/darkmode/evil.php HTTP/1.1',
        'http_request_referer': None,
        'http_request_user_agent': (
            'Mozilla/5.0 (Windows NT 7.1) AppleWebKit/534.30 (KHTML, like '
            'Gecko) Chrome/12.0.742.112 Safari/534.30'),
        'http_response_code': 200,
        'http_response_bytes': 1063,
        'ip_address': '192.168.0.2',
        'port_number': 443,
        'recorded_time': '2018-01-13T19:31:17+00:00',
        'remote_name': None,
        'server_name': 'plaso.log2timeline.net',
        'user_name': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 8)
    self.CheckEventData(event_data, expected_event_values)

    # Test common log format parser event with Kerberos user name
    expected_event_values = {
        'data_type': 'apache:access_log:entry',
        'http_request': 'GET / HTTP/1.1',
        'http_response_code': 200,
        'http_response_bytes': 8264,
        'ip_address': '192.168.0.64',
        'recorded_time': '2019-11-16T09:46:42+02:00',
        'remote_name': None,
        'user_name': 'pyllyukko@EXAMPLE.COM'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 13)
    self.CheckEventData(event_data, expected_event_values)

    # Test an extraction warning.
    generator = storage_writer.GetAttributeContainers(
        warnings.ExtractionWarning.CONTAINER_TYPE)

    test_warnings = list(generator)
    test_warning = test_warnings[0]

    expected_message = (
        'unable to parse log line: 8 "46.118.127.106 - - [20/May/2015:12:05:17 '
        '+0000] "GET /scripts/grok-py-test/co..."')
    self.assertEqual(test_warning.message, expected_message)
    self.assertEqual(test_warning.parser_chain, 'text/apache_access')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Sophos Anti-Virus log (SAV.txt) text parser plugin."""

import unittest

from plaso.parsers.text_plugins import sophos_av

from tests.parsers.text_plugins import test_lib


class SophosAVLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the Sophos Anti-Virus log (SAV.txt) text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = sophos_av.SophosAVLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['sav.txt'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2010-07-20T18:38:14',
        'data_type': 'sophos:av:log',
        'text': (
            'File "C:\\Documents and Settings\\Administrator\\Desktop\\'
            'sxl_test_50.com" belongs to virus/spyware \'LiveProtectTest\'.')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""YAML-based formatters file."""

import yaml

from plaso.formatters import interface
from plaso.lib import errors


class YAMLFormattersFile(object):
  """YAML-based formatters file.

  A YAML-based formatters file contains one or more event formatter
  definitions. An event formatter definition consists of:

  type: 'conditional'
  data_type: 'fs:stat'
  message:
  - '{display_name}'
  - 'Type: {file_entry_type}'
  - '({unallocated})'
  short_message:
  - '{filename}'
  short_source: 'FILE'
  source: 'File stat'

  Where:
  * type, defines the formatter data type, which can be "basic" or
    "conditional";
  * data_type, defines the corresponding event data type;
  * message, defines a list of message string pieces;
  * separator, defines the message and short message string pieces separator;
  * short_message, defines the short message string pieces;
  * short_source, defines the short source description;
  * source, defines the source description.
  """

  _SUPPORTED_KEYS = frozenset([
      'data_type',
      'boolean_helpers',
      'custom_helpers',
      'enumeration_helpers',
      'flags_helpers',
      'message',
      'separator',
      'short_message',
      'short_source',
      'source',
      'type'])

  def _ReadBooleanHelpers(self, formatter, boolean_helpers_definition_values):
    """Reads boolean helper definitions from a list.

    Args:
      formatter (EventFormatter): an event formatter.
      boolean_helpers_definition_values (list[dict[str, object]]):
          boolean helpers definition values.

    Raises:
      ParseError: if the format of the boolean helper definitions are incorrect.
    """
    for boolean_helper in boolean_helpers_definition_values:
      input_attribute = boolean_helper.get('input_attribute', None)
      if not input_attribute:
        raise errors.ParseError(
            'Invalid boolean helper missing input attribute.')

      output_attribute = boolean_helper.get('output_attribute', None)
      if not output_attribute:
        raise errors.ParseError(
            'Invalid boolean helper missing output attribute.')

      value_if_false = boolean_helper.get('value_if_false', None)
      value_if_true = boolean_helper.get('value_if_true', None)

      helper = interface.BooleanEventFormatterHelper(
          input_attribute=input_attribute, output_attribute=output_attribute,
          value_if_false=value_if_false, value_if_true=value_if_true)

      formatter.AddHelper(helper)

  def _ReadCustomHelpers(self, formatter, custom_helpers_definition_values):
    """Reads custom helper definitions from a list.

    Args:
      formatter (EventFormatter): an event formatter.
      custom_helpers_definition_values (list[dict[str, object]]):
          custom helpers definition values.

    Raises:
      ParseError: if the format of the custom helper definitions are incorrect.
    """
    for custom_helper in custom_helpers_definition_values:
      identifier = custom_helper.get('identifier', None)
      if not identifier:
        raise errors.ParseError(
            'Invalid custom helper missing identifier.')

      input_attribute = custom_helper.get('input_attribute', None)
      output_attribute = custom_helper.get('output_attribute', None)

      formatter.AddCustomHelper(
          identifier, input_attribute=input_attribute,
          output_attribute=output_attribute)

  def _ReadEnumerationHelpers(
      self, formatter, enumeration_helpers_definition_values):
    """Reads enumeration helper definitions from a list.

    Args:
      formatter (EventFormatter): an event formatter.
      enumeration_helpers_definition_values (list[dict[str, object]]):
          enumeration helpers definition values.

    Raises:
      ParseError: if the format of the enumeration helper definitions are
          incorrect.
    """
    for enumeration_helper in enumeration_helpers_definition_values:
      input_attribute = enumeration_helper.get('input_attribute', None)
      if not input_attribute:
        raise errors.ParseError(
            'Invalid enumeration helper missing input attribute.')

      output_attribute = enumeration_helper.get('output_attribute', None)
      if not output_attribute:
        raise errors.ParseError(
            'Invalid enumeration helper missing output attribute.')

      values = enumeration_helper.get('values', None)
      if not values:
        raise errors.ParseError('Invalid enumeration helper missing values.')

      default_value = enumeration_helper.get('default_value', None)

      helper = interface.EnumerationEventFormatterHelper(
          default=default_value, input_attribute=input_attribute,
          output_attribute=output_attribute, values=values)

      formatter.AddHelper(helper)

  def _ReadFlagsHelpers(self, formatter, flags_helpers_definition_values):
    """Reads flags helper definitions from a list.

    Args:
      formatter (EventFormatter): an event formatter.
      flags_helpers_definition_values (list[dict[str, object]]): flags helpers
          definition values.

    Raises:
      ParseError: if the format of the flags helper definitions are incorrect.
    """
    for flags_helper in flags_helpers_definition_values:
      input_attribute = flags_helper.get('input_attribute', None)
      if not input_attribute:
        raise errors.ParseError(
            'Invalid flags helper missing input attribute.')

      output_attribute = flags_helper.get('output_attribute', None)
      if not output_attribute:
        raise errors.ParseError(
            'Invalid flags helper missing output attribute.')

      values = flags_helper.get('values', None)
      if not values:
        raise errors.ParseError('Invalid flags helper missing values.')

      helper = interface.FlagsEventFormatterHelper(
          input_attribute=input_attribute, output_attribute=output_attribute,
          values=values)

      formatter.AddHelper(helper)

  def _ReadFormatterDefinition(self, formatter_definition_values):
    """Reads an event formatter definition from a dictionary.

    Args:
      formatter_definition_values (dict[str, object]): formatter definition
          values.

    Returns:
      EventFormatter: an event formatter.

    Raises:
      ParseError: if the format of the formatter definition is not set
          or incorrect.
    """
    if not formatter_definition_values:
      raise errors.ParseError('Missing formatter definition values.')

    different_keys = set(formatter_definition_values) - self._SUPPORTED_KEYS
    if different_keys:
      different_keys = ', '.join(different_keys)
      raise errors.ParseError('Undefined keys: {0:s}'.format(different_keys))

    formatter_type = formatter_definition_values.get('type', None)
    if not formatter_type:
      raise errors.ParseError(
          'Invalid event formatter definition missing type.')

    if formatter_type not in ('basic', 'conditional'):
      raise errors.ParseError(
          'Invalid event formatter definition unsupported type: {0!s}.'.format(
              formatter_type))

    data_type = formatter_definition_values.get('data_type', None)
    if not data_type:
      raise errors.ParseError(
          'Invalid event formatter definition missing data type.')

    message = formatter_definition_values.get('message', None)
    if not message:
      raise errors.ParseError(
          'Invalid event formatter definition: {0:s} missing message.'.format(
              data_type))

    short_message = formatter_definition_values.get('short_message', None)
    if not short_message:
      raise errors.ParseError((
          'Invalid event formatter definition: {0:s} missing short '
          'message.').format(data_type))

    short_source = formatter_definition_values.get('short_source', None)
    if not short_source:
      raise errors.ParseError((
          'Invalid event formatter definition: {0:s} missing short '
          'source.').format(data_type))

    source = formatter_definition_values.get('source', None)
    if not source:
      raise errors.ParseError(
          'Invalid event formatter definition: {0:s} missing source.'.format(
              data_type))

    if formatter_type == 'basic':
      formatter = interface.BasicEventFormatter(
          data_type=data_type, format_string=message,
          format_string_short=short_message)

    elif formatter_type == 'conditional':
      separator = formatter_definition_values.get('separator', None)
      formatter = interface.ConditionalEventFormatter(
          data_type=data_type, format_string_pieces=message,
          format_string_separator=separator,
          format_string_short_pieces=short_message)

    boolean_helpers = formatter_definition_values.get('boolean_helpers', [])
    self._ReadBooleanHelpers(formatter, boolean_helpers)

    custom_helpers = formatter_definition_values.get('custom_helpers', [])
    self._ReadCustomHelpers(formatter, custom_helpers)

    enumeration_helpers = formatter_definition_values.get(
        'enumeration_helpers', [])
    self._ReadEnumerationHelpers(formatter, enumeration_helpers)

    flags_helpers = formatter_definition_values.get('flags_helpers', [])
    self._ReadFlagsHelpers(formatter, flags_helpers)

    if short_source and source:
      formatter.source_mapping = (short_source, source)

    return formatter

  def _ReadFromFileObject(self, file_object):
    """Reads the event formatters from a file-like object.

    Args:
      file_object (file): formatters file-like object.

    Yields:
      EventFormatter: an event formatter.
    """
    yaml_generator = yaml.safe_load_all(file_object)

    for yaml_definition in yaml_generator:
      yield self._ReadFormatterDefinition(yaml_definition)

  def ReadFromFile(self, path):
    """Reads the event formatters from a YAML file.

    Args:
      path (str): path to a formatters file.

    Yields:
      EventFormatter: an event formatter.
    """
    with open(path, 'r', encoding='utf-8') as file_object:
      for yaml_definition in self._ReadFromFileObject(file_object):
        yield yaml_definition



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iMessage plugin."""

import unittest

from plaso.parsers.sqlite_plugins import imessage

from tests.parsers.sqlite_plugins import test_lib


class IMessageTest(test_lib.SQLitePluginTestCase):
  """Tests for the iMessage database plugin."""

  def testProcess(self):
    """Test the Process function on a iMessage chat.db file."""
    plugin = imessage.IMessagePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['imessage_chat.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 10)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'creation_time': '2015-11-30T10:48:40.000000+00:00',
        'data_type': 'imessage:event:chat',
        'imessage_id': 'xxxxxx2015@icloud.com',
        'message_type': 0,
        'read_receipt': 1,
        'service': 'iMessage',
        'text': 'Did you try to send me a message?'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 7)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS TimeMachine plist plugin."""

import unittest

from plaso.parsers.plist_plugins import time_machine

from tests.parsers.plist_plugins import test_lib


class MacOSTimeMachinePlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the MacOS TimeMachine plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'com.apple.TimeMachine.plist'

    plugin = time_machine.MacOSTimeMachinePlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'backup_alias': 'BackUpFast',
        'data_type': 'macos:time_machine:backup',
        'destination_identifier': '5B33C22B-A4A1-4024-A2F5-C9979C4AAAAA',
        'snapshot_times': [
            '2013-09-14T13:24:11.000000+00:00',
            '2013-09-25T08:40:55.000000+00:00',
            '2013-10-03T14:24:36.000000+00:00',
            '2013-10-16T00:32:18.000000+00:00',
            '2013-10-24T20:51:30.000000+00:00',
            '2013-11-02T00:22:19.000000+00:00',
            '2013-11-10T13:27:00.000000+00:00',
            '2013-11-22T14:35:14.000000+00:00',
            '2013-12-05T17:51:51.000000+00:00',
            '2013-12-10T15:37:32.000000+00:00',
            '2013-12-22T14:38:11.000000+00:00',
            '2014-01-04T13:09:10.000000+00:00',
            '2014-01-04T13:38:38.000000+00:00']}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The multi-process extraction worker process."""

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.resolver import context
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.engine import worker
from plaso.lib import definitions
from plaso.lib import errors
from plaso.multi_process import logger
from plaso.multi_process import plaso_queue
from plaso.multi_process import task_process
from plaso.parsers import mediator as parsers_mediator


class ExtractionWorkerProcess(task_process.MultiProcessTaskProcess):
  """Multi-processing extraction worker process."""

  # Maximum number of dfVFS file system objects to cache in the worker process.
  _FILE_SYSTEM_CACHE_SIZE = 3

  def __init__(
      self, task_queue, processing_configuration, system_configurations,
      registry_find_specs, **kwargs):
    """Initializes an extraction worker process.

    Non-specified keyword arguments (kwargs) are directly passed to
    multiprocessing.Process.

    Args:
      task_queue (PlasoQueue): task queue.
      processing_configuration (ProcessingConfiguration): processing
          configuration.
      system_configurations (list[SystemConfigurationArtifact]): system
          configurations.
     registry_find_specs (list[dfwinreg.FindSpec]): Windows Registry find
         specifications.
      kwargs: keyword arguments to pass to multiprocessing.Process.
    """
    super(ExtractionWorkerProcess, self).__init__(
        processing_configuration, **kwargs)
    self._abort = False
    self._buffer_size = 0
    self._current_display_name = ''
    self._extraction_worker = None
    self._file_system_cache = []
    self._number_of_consumed_sources = 0
    self._parser_mediator = None
    self._registry_find_specs = registry_find_specs
    self._resolver_context = None
    self._status = definitions.STATUS_INDICATOR_INITIALIZED
    self._task = None
    self._task_queue = task_queue
    self._system_configurations = system_configurations

  def _CacheFileSystem(self, file_system):
    """Caches a dfVFS file system object.

    Keeping and additional reference to a dfVFS file system object causes the
    object to remain cached in the resolver context. This minimizes the number
    times the file system is re-opened.

    Args:
      file_system (dfvfs.FileSystem): file system.
    """
    if file_system not in self._file_system_cache:
      if len(self._file_system_cache) == self._FILE_SYSTEM_CACHE_SIZE:
        self._file_system_cache.pop(0)
      self._file_system_cache.append(file_system)

    elif len(self._file_system_cache) == self._FILE_SYSTEM_CACHE_SIZE:
      # Move the file system to the end of the list to preserve the most
      # recently file system object.
      self._file_system_cache.remove(file_system)
      self._file_system_cache.append(file_system)

  def _CreateParserMediator(
      self, resolver_context, processing_configuration, system_configurations):
    """Creates a parser mediator.

    Args:
      resolver_context (dfvfs.Context): resolver context.
      processing_configuration (ProcessingConfiguration): processing
          configuration.
      system_configurations (list[SystemConfigurationArtifact]): system
          configurations.

    Returns:
      ParserMediator: parser mediator.
    """
    mediator = parsers_mediator.ParserMediator(
        registry_find_specs=self._registry_find_specs,
        resolver_context=resolver_context,
        system_configurations=system_configurations)

    mediator.SetExtractWinEvtResources(
        processing_configuration.extraction.extract_winevt_resources)
    mediator.SetPreferredCodepage(processing_configuration.preferred_codepage)
    mediator.SetPreferredLanguage(processing_configuration.preferred_language)
    mediator.SetTemporaryDirectory(processing_configuration.temporary_directory)

    return mediator

  def _GetStatus(self):
    """Retrieves status information.

    Returns:
      dict[str, object]: status attributes, indexed by name.
    """
    if self._parser_mediator:
      number_of_produced_event_data = (
          self._parser_mediator.number_of_produced_event_data)
      number_of_produced_sources = (
          self._parser_mediator.number_of_produced_event_sources)
    else:
      number_of_produced_event_data = None
      number_of_produced_sources = None

    if self._extraction_worker and self._parser_mediator:
      last_activity_timestamp = max(
          self._extraction_worker.last_activity_timestamp,
          self._parser_mediator.last_activity_timestamp)
      processing_status = self._extraction_worker.processing_status
    else:
      last_activity_timestamp = 0.0
      processing_status = self._status

    task_identifier = getattr(self._task, 'identifier', '')

    if self._process_information:
      used_memory = self._process_information.GetUsedMemory() or 0
    else:
      used_memory = 0

    if self._memory_profiler:
      self._memory_profiler.Sample('main', used_memory)

    # XML RPC does not support integer values > 2 GiB so we format them
    # as a string.
    used_memory = '{0:d}'.format(used_memory)

    status = {
        'display_name': self._current_display_name,
        'identifier': self._name,
        'last_activity_timestamp': last_activity_timestamp,
        'number_of_consumed_event_data': None,
        'number_of_consumed_event_tags': None,
        'number_of_consumed_events': None,
        'number_of_consumed_sources': self._number_of_consumed_sources,
        'number_of_produced_event_data': number_of_produced_event_data,
        'number_of_produced_event_tags': None,
        'number_of_produced_events': None,
        'number_of_produced_sources': number_of_produced_sources,
        'processing_status': processing_status,
        'task_identifier': task_identifier,
        'used_memory': used_memory}

    return status

  def _Main(self):
    """The main loop."""
    # We need a resolver context per process to prevent multi processing
    # issues with file objects stored in images.
    self._resolver_context = context.Context()

    for credential_configuration in self._processing_configuration.credentials:
      path_spec_resolver.Resolver.key_chain.SetCredential(
          credential_configuration.path_spec,
          credential_configuration.credential_type,
          credential_configuration.credential_data)

    self._parser_mediator = self._CreateParserMediator(
        self._resolver_context, self._processing_configuration,
        self._system_configurations)

    # We need to initialize the parser and hasher objects after the process
    # has forked otherwise on Windows the "fork" will fail with
    # a PickleError for Python modules that cannot be pickled.
    self._extraction_worker = worker.EventExtractionWorker(
        parser_filter_expression=(
            self._processing_configuration.parser_filter_expression))

    self._extraction_worker.SetExtractionConfiguration(
        self._processing_configuration.extraction)

    self._parser_mediator.StartProfiling(
        self._processing_configuration.profiling, self._name,
        self._process_information)
    self._StartProfiling(self._processing_configuration.profiling)

    if self._analyzers_profiler:
      self._extraction_worker.SetAnalyzersProfiler(self._analyzers_profiler)

    if self._processing_profiler:
      self._extraction_worker.SetProcessingProfiler(self._processing_profiler)

    logger.debug('Worker: {0!s} (PID: {1:d}) started.'.format(
        self._name, self._pid))

    self._status = definitions.STATUS_INDICATOR_RUNNING

    try:
      logger.debug('{0!s} (PID: {1:d}) started monitoring task queue.'.format(
          self._name, self._pid))

      while not self._abort:
        try:
          task = self._task_queue.PopItem()
        except (errors.QueueClose, errors.QueueEmpty) as exception:
          logger.debug('ConsumeItems exiting with exception: {0!s}.'.format(
              type(exception)))
          break

        if isinstance(task, plaso_queue.QueueAbort):
          logger.debug('ConsumeItems exiting, dequeued QueueAbort object.')
          break

        self._ProcessTask(task)

      logger.debug('{0!s} (PID: {1:d}) stopped monitoring task queue.'.format(
          self._name, self._pid))

    # All exceptions need to be caught here to prevent the process
    # from being killed by an uncaught exception.
    except Exception as exception:  # pylint: disable=broad-except
      logger.warning(
          'Unhandled exception in process: {0!s} (PID: {1:d}).'.format(
              self._name, self._pid))
      logger.exception(exception)

      self._abort = True

    if self._analyzers_profiler:
      self._extraction_worker.SetAnalyzersProfiler(None)

    if self._processing_profiler:
      self._extraction_worker.SetProcessingProfiler(None)

    self._StopProfiling()
    self._parser_mediator.StopProfiling()

    self._extraction_worker = None
    self._file_system_cache = []
    self._parser_mediator = None
    self._resolver_context = None

    if self._abort:
      self._status = definitions.STATUS_INDICATOR_ABORTED
    else:
      self._status = definitions.STATUS_INDICATOR_COMPLETED

    logger.debug('Worker: {0!s} (PID: {1:d}) stopped.'.format(
        self._name, self._pid))

    try:
      self._task_queue.Close(abort=self._abort)
    except errors.QueueAlreadyClosed:
      logger.error('Queue for {0:s} was already closed.'.format(self.name))

  def _ProcessPathSpec(self, extraction_worker, parser_mediator, path_spec):
    """Processes a path specification.

    Args:
      extraction_worker (worker.ExtractionWorker): extraction worker.
      parser_mediator (ParserMediator): parser mediator.
      path_spec (dfvfs.PathSpec): path specification.
    """
    self._current_display_name = parser_mediator.GetDisplayNameForPathSpec(
        path_spec)

    try:
      file_entry = path_spec_resolver.Resolver.OpenFileEntry(
          path_spec, resolver_context=parser_mediator.resolver_context)
      if file_entry is None:
        logger.warning('Unable to open file entry: {0:s}'.format(
            self._current_display_name))
        return

      if (path_spec and not path_spec.IsSystemLevel() and
          path_spec.type_indicator != dfvfs_definitions.TYPE_INDICATOR_GZIP):
        file_system = file_entry.GetFileSystem()
        self._CacheFileSystem(file_system)

      extraction_worker.ProcessFileEntry(parser_mediator, file_entry)

    except Exception as exception:  # pylint: disable=broad-except
      parser_mediator.ProduceExtractionWarning((
          'unable to process path specification with error: '
          '{0!s}').format(exception), path_spec=path_spec)

      if self._processing_configuration.debug_output:
        logger.warning((
            'Unhandled exception while processing path specification: '
            '{0:s}.').format(self._current_display_name))
        logger.exception(exception)

  def _ProcessTask(self, task):
    """Processes a task.

    Args:
      task (Task): task.
    """
    logger.debug('Started processing task: {0:s}.'.format(task.identifier))

    if self._tasks_profiler:
      self._tasks_profiler.Sample(task, 'processing_started')

    task.storage_format = self._processing_configuration.task_storage_format

    self._task = task

    task_storage_writer = self._storage_factory.CreateTaskStorageWriter(
        self._processing_configuration.task_storage_format)

    if self._serializers_profiler:
      task_storage_writer.SetSerializersProfiler(self._serializers_profiler)

    if self._storage_profiler:
      task_storage_writer.SetStorageProfiler(self._storage_profiler)

    self._parser_mediator.SetStorageWriter(task_storage_writer)

    storage_file_path = self._GetTaskStorageFilePath(
        self._processing_configuration.task_storage_format, task)
    task_storage_writer.Open(
        path=storage_file_path, session_identifier=task.session_identifier,
        task_identifier=task.identifier)

    try:
      task_storage_writer.AddAttributeContainer(task)

      # TODO: add support for more task types.
      self._ProcessPathSpec(
          self._extraction_worker, self._parser_mediator, task.path_spec)
      self._number_of_consumed_sources += 1

    finally:
      task.aborted = self._abort
      task_storage_writer.UpdateAttributeContainer(task)

      task_storage_writer.Close()

    self._parser_mediator.SetStorageWriter(None)

    try:
      self._FinalizeTaskStorageWriter(
          self._processing_configuration.task_storage_format, task)
    except IOError:
      pass

    self._task = None

    if self._tasks_profiler:
      self._tasks_profiler.Sample(task, 'processing_completed')

    logger.debug('Completed processing task: {0:s}.'.format(task.identifier))

  def SignalAbort(self):
    """Signals the process to abort."""
    self._abort = True
    if self._extraction_worker:
      self._extraction_worker.SignalAbort()
    if self._parser_mediator:
      self._parser_mediator.SignalAbort()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS Document Versions plugin."""

import unittest

from plaso.parsers.sqlite_plugins import macos_document_versions

from tests.parsers.sqlite_plugins import test_lib


class MacOSDocumentVersionsTest(test_lib.SQLitePluginTestCase):
  """Tests for the MacOS Document Versions plugin."""

  def testProcess(self):
    """Tests the Process function on a MacOS Document Versions file."""
    plugin = macos_document_versions.MacOSDocumentVersionsPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['document_versions.sql'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'creation_time': '2014-01-21T02:03:00+00:00',
        'data_type': 'macos:document_versions:file',
        'name': 'Spain is beautiful.rtf',
        'last_seen_time': '2014-01-21T02:13:01+00:00',
        'path': '/Users/moxilo/Documents',
        'user_sid': '501',
        'version_path': (
            '/.DocumentRevisions-V100/PerUID/501/1/com.apple.documentVersions/'
            '08CFEB5A-5CDA-486F-AED5-EA35BF3EE4C2.rtf')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()
